"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def query(
    hub,
    ctx,
    table_name: str,
    index_name: str = None,
    select: str = None,
    attributes_to_get: List = None,
    limit: int = None,
    consistent_read: bool = None,
    key_conditions: Dict = None,
    query_filter: Dict = None,
    conditional_operator: str = None,
    scan_index_forward: bool = None,
    exclusive_start_key: Dict = None,
    return_consumed_capacity: str = None,
    projection_expression: str = None,
    filter_expression: str = None,
    key_condition_expression: str = None,
    expression_attribute_names: Dict = None,
    expression_attribute_values: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    The Query operation finds items based on primary key values. You can query any table or secondary index that has a composite primary key (a partition key and a sort key).  Use the KeyConditionExpression parameter to provide a specific value for the partition key. The Query operation will return all of the items from the table or index with that partition key value. You can optionally narrow the scope of the Query operation by specifying a sort key value and a comparison operator in KeyConditionExpression. To further refine the Query results, you can optionally provide a FilterExpression. A FilterExpression determines which items within the results should be returned to you. All of the other results are discarded.   A Query operation always returns a result set. If no matching items are found, the result set will be empty. Queries that do not return results consume the minimum number of read capacity units for that type of read operation.    DynamoDB calculates the number of read capacity units consumed based on item size, not on the amount of data that is returned to an application. The number of capacity units consumed will be the same whether you request all of the attributes (the default behavior) or just some of them (using a projection expression). The number will also be the same whether or not you use a FilterExpression.    Query results are always sorted by the sort key value. If the data type of the sort key is Number, the results are returned in numeric order; otherwise, the results are returned in order of UTF-8 bytes. By default, the sort order is ascending. To reverse the order, set the ScanIndexForward parameter to false.   A single Query operation will read up to the maximum number of items set (if using the Limit parameter) or a maximum of 1 MB of data and then apply any filtering to the results using FilterExpression. If LastEvaluatedKey is present in the response, you will need to paginate the result set. For more information, see Paginating the Results in the Amazon DynamoDB Developer Guide.   FilterExpression is applied after a Query finishes, but before the results are returned. A FilterExpression cannot contain partition key or sort key attributes. You need to specify those attributes in the KeyConditionExpression.    A Query operation can return an empty result set and a LastEvaluatedKey if all the items read for the page of results are filtered out.   You can query a table, a local secondary index, or a global secondary index. For a query on a table or on a local secondary index, you can set the ConsistentRead parameter to true and obtain a strongly consistent result. Global secondary indexes support eventually consistent reads only, so do not specify ConsistentRead when querying a global secondary index.

    Args:
        table_name(str): The name of the table containing the requested items.
        index_name(str, optional): The name of an index to query. This index can be any local secondary index or global secondary index on the table. Note that if you use the IndexName parameter, you must also provide TableName. . Defaults to None.
        select(str, optional): The attributes to be returned in the result. You can retrieve all item attributes, specific item attributes, the count of matching items, or in the case of an index, some or all of the attributes projected into the index.    ALL_ATTRIBUTES - Returns all of the item attributes from the specified table or index. If you query a local secondary index, then for each matching item in the index, DynamoDB fetches the entire item from the parent table. If the index is configured to project all item attributes, then all of the data can be obtained from the local secondary index, and no fetching is required.    ALL_PROJECTED_ATTRIBUTES - Allowed only when querying an index. Retrieves all attributes that have been projected into the index. If the index is configured to project all attributes, this return value is equivalent to specifying ALL_ATTRIBUTES.    COUNT - Returns the number of matching items, rather than the matching items themselves.    SPECIFIC_ATTRIBUTES - Returns only the attributes listed in AttributesToGet. This return value is equivalent to specifying AttributesToGet without specifying any value for Select. If you query or scan a local secondary index and request only attributes that are projected into that index, the operation will read only the index and not the table. If any of the requested attributes are not projected into the local secondary index, DynamoDB fetches each of these attributes from the parent table. This extra fetching incurs additional throughput cost and latency. If you query or scan a global secondary index, you can only request attributes that are projected into the index. Global secondary index queries cannot fetch attributes from the parent table.   If neither Select nor AttributesToGet are specified, DynamoDB defaults to ALL_ATTRIBUTES when accessing a table, and ALL_PROJECTED_ATTRIBUTES when accessing an index. You cannot use both Select and AttributesToGet together in a single request, unless the value for Select is SPECIFIC_ATTRIBUTES. (This usage is equivalent to specifying AttributesToGet without any value for Select.)  If you use the ProjectionExpression parameter, then the value for Select can only be SPECIFIC_ATTRIBUTES. Any other value for Select will return an error. . Defaults to None.
        attributes_to_get(List, optional): This is a legacy parameter. Use ProjectionExpression instead. For more information, see AttributesToGet in the Amazon DynamoDB Developer Guide. Defaults to None.
        limit(int, optional): The maximum number of items to evaluate (not necessarily the number of matching items). If DynamoDB processes the number of items up to the limit while processing the results, it stops the operation and returns the matching values up to that point, and a key in LastEvaluatedKey to apply in a subsequent operation, so that you can pick up where you left off. Also, if the processed dataset size exceeds 1 MB before DynamoDB reaches this limit, it stops the operation and returns the matching values up to the limit, and a key in LastEvaluatedKey to apply in a subsequent operation to continue the operation. For more information, see Query and Scan in the Amazon DynamoDB Developer Guide. Defaults to None.
        consistent_read(bool, optional): Determines the read consistency model: If set to true, then the operation uses strongly consistent reads; otherwise, the operation uses eventually consistent reads. Strongly consistent reads are not supported on global secondary indexes. If you query a global secondary index with ConsistentRead set to true, you will receive a ValidationException. Defaults to None.
        key_conditions(Dict, optional): This is a legacy parameter. Use KeyConditionExpression instead. For more information, see KeyConditions in the Amazon DynamoDB Developer Guide. Defaults to None.
        query_filter(Dict, optional): This is a legacy parameter. Use FilterExpression instead. For more information, see QueryFilter in the Amazon DynamoDB Developer Guide. Defaults to None.
        conditional_operator(str, optional): This is a legacy parameter. Use FilterExpression instead. For more information, see ConditionalOperator in the Amazon DynamoDB Developer Guide. Defaults to None.
        scan_index_forward(bool, optional): Specifies the order for index traversal: If true (default), the traversal is performed in ascending order; if false, the traversal is performed in descending order.  Items with the same partition key value are stored in sorted order by sort key. If the sort key data type is Number, the results are stored in numeric order. For type String, the results are stored in order of UTF-8 bytes. For type Binary, DynamoDB treats each byte of the binary data as unsigned. If ScanIndexForward is true, DynamoDB returns the results in the order in which they are stored (by sort key value). This is the default behavior. If ScanIndexForward is false, DynamoDB reads the results in reverse order by sort key value, and then returns the results to the client. Defaults to None.
        exclusive_start_key(Dict, optional): The primary key of the first item that this operation will evaluate. Use the value that was returned for LastEvaluatedKey in the previous operation. The data type for ExclusiveStartKey must be String, Number, or Binary. No set data types are allowed. Defaults to None.
        return_consumed_capacity(str, optional): Determines the level of detail about provisioned throughput consumption that is returned in the response:    INDEXES - The response includes the aggregate ConsumedCapacity for the operation, together with ConsumedCapacity for each table and secondary index that was accessed. Note that some operations, such as GetItem and BatchGetItem, do not access any indexes at all. In these cases, specifying INDEXES will only return ConsumedCapacity information for table(s).    TOTAL - The response includes only the aggregate ConsumedCapacity for the operation.    NONE - No ConsumedCapacity details are included in the response.  . Defaults to None.
        projection_expression(str, optional): A string that identifies one or more attributes to retrieve from the table. These attributes can include scalars, sets, or elements of a JSON document. The attributes in the expression must be separated by commas. If no attribute names are specified, then all attributes will be returned. If any of the requested attributes are not found, they will not appear in the result. For more information, see Accessing Item Attributes in the Amazon DynamoDB Developer Guide. Defaults to None.
        filter_expression(str, optional): A string that contains conditions that DynamoDB applies after the Query operation, but before the data is returned to you. Items that do not satisfy the FilterExpression criteria are not returned. A FilterExpression does not allow key attributes. You cannot define a filter expression based on a partition key or a sort key.  A FilterExpression is applied after the items have already been read; the process of filtering does not consume any additional read capacity units.  For more information, see Filter Expressions in the Amazon DynamoDB Developer Guide. Defaults to None.
        key_condition_expression(str, optional): The condition that specifies the key values for items to be retrieved by the Query action. The condition must perform an equality test on a single partition key value. The condition can optionally perform one of several comparison tests on a single sort key value. This allows Query to retrieve one item with a given partition key value and sort key value, or several items that have the same partition key value but different sort key values. The partition key equality test is required, and must be specified in the following format:  partitionKeyName = :partitionkeyval  If you also want to provide a condition for the sort key, it must be combined using AND with the condition for the sort key. Following is an example, using the = comparison operator for the sort key:  partitionKeyName = :partitionkeyval AND sortKeyName = :sortkeyval  Valid comparisons for the sort key condition are as follows:    sortKeyName = :sortkeyval - true if the sort key value is equal to :sortkeyval.    sortKeyName < :sortkeyval - true if the sort key value is less than :sortkeyval.    sortKeyName <= :sortkeyval - true if the sort key value is less than or equal to :sortkeyval.    sortKeyName > :sortkeyval - true if the sort key value is greater than :sortkeyval.    sortKeyName >=  :sortkeyval - true if the sort key value is greater than or equal to :sortkeyval.    sortKeyName BETWEEN :sortkeyval1 AND :sortkeyval2 - true if the sort key value is greater than or equal to :sortkeyval1, and less than or equal to :sortkeyval2.    begins_with ( sortKeyName, :sortkeyval ) - true if the sort key value begins with a particular operand. (You cannot use this function with a sort key that is of type Number.) Note that the function name begins_with is case-sensitive.   Use the ExpressionAttributeValues parameter to replace tokens such as :partitionval and :sortval with actual values at runtime. You can optionally use the ExpressionAttributeNames parameter to replace the names of the partition key and sort key with placeholder tokens. This option might be necessary if an attribute name conflicts with a DynamoDB reserved word. For example, the following KeyConditionExpression parameter causes an error because Size is a reserved word:    Size = :myval    To work around this, define a placeholder (such a #S) to represent the attribute name Size. KeyConditionExpression then is as follows:    #S = :myval    For a list of reserved words, see Reserved Words in the Amazon DynamoDB Developer Guide. For more information on ExpressionAttributeNames and ExpressionAttributeValues, see Using Placeholders for Attribute Names and Values in the Amazon DynamoDB Developer Guide. Defaults to None.
        expression_attribute_names(Dict, optional): One or more substitution tokens for attribute names in an expression. The following are some use cases for using ExpressionAttributeNames:   To access an attribute whose name conflicts with a DynamoDB reserved word.   To create a placeholder for repeating occurrences of an attribute name in an expression.   To prevent special characters in an attribute name from being misinterpreted in an expression.   Use the # character in an expression to dereference an attribute name. For example, consider the following attribute name:    Percentile    The name of this attribute conflicts with a reserved word, so it cannot be used directly in an expression. (For the complete list of reserved words, see Reserved Words in the Amazon DynamoDB Developer Guide). To work around this, you could specify the following for ExpressionAttributeNames:    {"#P":"Percentile"}    You could then use this substitution in an expression, as in this example:    #P = :val     Tokens that begin with the : character are expression attribute values, which are placeholders for the actual value at runtime.  For more information on expression attribute names, see Specifying Item Attributes in the Amazon DynamoDB Developer Guide. Defaults to None.
        expression_attribute_values(Dict, optional): One or more values that can be substituted in an expression. Use the : (colon) character in an expression to dereference an attribute value. For example, suppose that you wanted to check whether the value of the ProductStatus attribute was one of the following:   Available | Backordered | Discontinued  You would first need to specify ExpressionAttributeValues as follows:  { ":avail":{"S":"Available"}, ":back":{"S":"Backordered"}, ":disc":{"S":"Discontinued"} }  You could then use these values in an expression, such as this:  ProductStatus IN (:avail, :back, :disc)  For more information on expression attribute values, see Specifying Conditions in the Amazon DynamoDB Developer Guide. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dynamodb.init.query(ctx, table_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dynamodb.init.query table_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dynamodb",
        operation="query",
        op_kwargs={
            "TableName": table_name,
            "IndexName": index_name,
            "Select": select,
            "AttributesToGet": attributes_to_get,
            "Limit": limit,
            "ConsistentRead": consistent_read,
            "KeyConditions": key_conditions,
            "QueryFilter": query_filter,
            "ConditionalOperator": conditional_operator,
            "ScanIndexForward": scan_index_forward,
            "ExclusiveStartKey": exclusive_start_key,
            "ReturnConsumedCapacity": return_consumed_capacity,
            "ProjectionExpression": projection_expression,
            "FilterExpression": filter_expression,
            "KeyConditionExpression": key_condition_expression,
            "ExpressionAttributeNames": expression_attribute_names,
            "ExpressionAttributeValues": expression_attribute_values,
        },
    )


async def scan(
    hub,
    ctx,
    table_name: str,
    index_name: str = None,
    attributes_to_get: List = None,
    limit: int = None,
    select: str = None,
    scan_filter: Dict = None,
    conditional_operator: str = None,
    exclusive_start_key: Dict = None,
    return_consumed_capacity: str = None,
    total_segments: int = None,
    segment: int = None,
    projection_expression: str = None,
    filter_expression: str = None,
    expression_attribute_names: Dict = None,
    expression_attribute_values: Dict = None,
    consistent_read: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    The Scan operation returns one or more items and item attributes by accessing every item in a table or a secondary index. To have DynamoDB return fewer items, you can provide a FilterExpression operation. If the total number of scanned items exceeds the maximum dataset size limit of 1 MB, the scan stops and results are returned to the user as a LastEvaluatedKey value to continue the scan in a subsequent operation. The results also include the number of items exceeding the limit. A scan can result in no table data meeting the filter criteria.  A single Scan operation reads up to the maximum number of items set (if using the Limit parameter) or a maximum of 1 MB of data and then apply any filtering to the results using FilterExpression. If LastEvaluatedKey is present in the response, you need to paginate the result set. For more information, see Paginating the Results in the Amazon DynamoDB Developer Guide.   Scan operations proceed sequentially; however, for faster performance on a large table or secondary index, applications can request a parallel Scan operation by providing the Segment and TotalSegments parameters. For more information, see Parallel Scan in the Amazon DynamoDB Developer Guide.  Scan uses eventually consistent reads when accessing the data in a table; therefore, the result set might not include the changes to data in the table immediately before the operation began. If you need a consistent copy of the data, as of the time that the Scan begins, you can set the ConsistentRead parameter to true.

    Args:
        table_name(str): The name of the table containing the requested items; or, if you provide IndexName, the name of the table to which that index belongs.
        index_name(str, optional): The name of a secondary index to scan. This index can be any local secondary index or global secondary index. Note that if you use the IndexName parameter, you must also provide TableName. Defaults to None.
        attributes_to_get(List, optional): This is a legacy parameter. Use ProjectionExpression instead. For more information, see AttributesToGet in the Amazon DynamoDB Developer Guide. Defaults to None.
        limit(int, optional): The maximum number of items to evaluate (not necessarily the number of matching items). If DynamoDB processes the number of items up to the limit while processing the results, it stops the operation and returns the matching values up to that point, and a key in LastEvaluatedKey to apply in a subsequent operation, so that you can pick up where you left off. Also, if the processed dataset size exceeds 1 MB before DynamoDB reaches this limit, it stops the operation and returns the matching values up to the limit, and a key in LastEvaluatedKey to apply in a subsequent operation to continue the operation. For more information, see Working with Queries in the Amazon DynamoDB Developer Guide. Defaults to None.
        select(str, optional): The attributes to be returned in the result. You can retrieve all item attributes, specific item attributes, the count of matching items, or in the case of an index, some or all of the attributes projected into the index.    ALL_ATTRIBUTES - Returns all of the item attributes from the specified table or index. If you query a local secondary index, then for each matching item in the index, DynamoDB fetches the entire item from the parent table. If the index is configured to project all item attributes, then all of the data can be obtained from the local secondary index, and no fetching is required.    ALL_PROJECTED_ATTRIBUTES - Allowed only when querying an index. Retrieves all attributes that have been projected into the index. If the index is configured to project all attributes, this return value is equivalent to specifying ALL_ATTRIBUTES.    COUNT - Returns the number of matching items, rather than the matching items themselves.    SPECIFIC_ATTRIBUTES - Returns only the attributes listed in AttributesToGet. This return value is equivalent to specifying AttributesToGet without specifying any value for Select. If you query or scan a local secondary index and request only attributes that are projected into that index, the operation reads only the index and not the table. If any of the requested attributes are not projected into the local secondary index, DynamoDB fetches each of these attributes from the parent table. This extra fetching incurs additional throughput cost and latency. If you query or scan a global secondary index, you can only request attributes that are projected into the index. Global secondary index queries cannot fetch attributes from the parent table.   If neither Select nor AttributesToGet are specified, DynamoDB defaults to ALL_ATTRIBUTES when accessing a table, and ALL_PROJECTED_ATTRIBUTES when accessing an index. You cannot use both Select and AttributesToGet together in a single request, unless the value for Select is SPECIFIC_ATTRIBUTES. (This usage is equivalent to specifying AttributesToGet without any value for Select.)  If you use the ProjectionExpression parameter, then the value for Select can only be SPECIFIC_ATTRIBUTES. Any other value for Select will return an error. . Defaults to None.
        scan_filter(Dict, optional): This is a legacy parameter. Use FilterExpression instead. For more information, see ScanFilter in the Amazon DynamoDB Developer Guide. Defaults to None.
        conditional_operator(str, optional): This is a legacy parameter. Use FilterExpression instead. For more information, see ConditionalOperator in the Amazon DynamoDB Developer Guide. Defaults to None.
        exclusive_start_key(Dict, optional): The primary key of the first item that this operation will evaluate. Use the value that was returned for LastEvaluatedKey in the previous operation. The data type for ExclusiveStartKey must be String, Number or Binary. No set data types are allowed. In a parallel scan, a Scan request that includes ExclusiveStartKey must specify the same segment whose previous Scan returned the corresponding value of LastEvaluatedKey. Defaults to None.
        return_consumed_capacity(str, optional): Determines the level of detail about provisioned throughput consumption that is returned in the response:    INDEXES - The response includes the aggregate ConsumedCapacity for the operation, together with ConsumedCapacity for each table and secondary index that was accessed. Note that some operations, such as GetItem and BatchGetItem, do not access any indexes at all. In these cases, specifying INDEXES will only return ConsumedCapacity information for table(s).    TOTAL - The response includes only the aggregate ConsumedCapacity for the operation.    NONE - No ConsumedCapacity details are included in the response.  . Defaults to None.
        total_segments(int, optional): For a parallel Scan request, TotalSegments represents the total number of segments into which the Scan operation will be divided. The value of TotalSegments corresponds to the number of application workers that will perform the parallel scan. For example, if you want to use four application threads to scan a table or an index, specify a TotalSegments value of 4. The value for TotalSegments must be greater than or equal to 1, and less than or equal to 1000000. If you specify a TotalSegments value of 1, the Scan operation will be sequential rather than parallel. If you specify TotalSegments, you must also specify Segment. Defaults to None.
        segment(int, optional): For a parallel Scan request, Segment identifies an individual segment to be scanned by an application worker. Segment IDs are zero-based, so the first segment is always 0. For example, if you want to use four application threads to scan a table or an index, then the first thread specifies a Segment value of 0, the second thread specifies 1, and so on. The value of LastEvaluatedKey returned from a parallel Scan request must be used as ExclusiveStartKey with the same segment ID in a subsequent Scan operation. The value for Segment must be greater than or equal to 0, and less than the value provided for TotalSegments. If you provide Segment, you must also provide TotalSegments. Defaults to None.
        projection_expression(str, optional): A string that identifies one or more attributes to retrieve from the specified table or index. These attributes can include scalars, sets, or elements of a JSON document. The attributes in the expression must be separated by commas. If no attribute names are specified, then all attributes will be returned. If any of the requested attributes are not found, they will not appear in the result. For more information, see Specifying Item Attributes in the Amazon DynamoDB Developer Guide. Defaults to None.
        filter_expression(str, optional): A string that contains conditions that DynamoDB applies after the Scan operation, but before the data is returned to you. Items that do not satisfy the FilterExpression criteria are not returned.  A FilterExpression is applied after the items have already been read; the process of filtering does not consume any additional read capacity units.  For more information, see Filter Expressions in the Amazon DynamoDB Developer Guide. Defaults to None.
        expression_attribute_names(Dict, optional): One or more substitution tokens for attribute names in an expression. The following are some use cases for using ExpressionAttributeNames:   To access an attribute whose name conflicts with a DynamoDB reserved word.   To create a placeholder for repeating occurrences of an attribute name in an expression.   To prevent special characters in an attribute name from being misinterpreted in an expression.   Use the # character in an expression to dereference an attribute name. For example, consider the following attribute name:    Percentile    The name of this attribute conflicts with a reserved word, so it cannot be used directly in an expression. (For the complete list of reserved words, see Reserved Words in the Amazon DynamoDB Developer Guide). To work around this, you could specify the following for ExpressionAttributeNames:    {"#P":"Percentile"}    You could then use this substitution in an expression, as in this example:    #P = :val     Tokens that begin with the : character are expression attribute values, which are placeholders for the actual value at runtime.  For more information on expression attribute names, see Specifying Item Attributes in the Amazon DynamoDB Developer Guide. Defaults to None.
        expression_attribute_values(Dict, optional): One or more values that can be substituted in an expression. Use the : (colon) character in an expression to dereference an attribute value. For example, suppose that you wanted to check whether the value of the ProductStatus attribute was one of the following:   Available | Backordered | Discontinued  You would first need to specify ExpressionAttributeValues as follows:  { ":avail":{"S":"Available"}, ":back":{"S":"Backordered"}, ":disc":{"S":"Discontinued"} }  You could then use these values in an expression, such as this:  ProductStatus IN (:avail, :back, :disc)  For more information on expression attribute values, see Condition Expressions in the Amazon DynamoDB Developer Guide. Defaults to None.
        consistent_read(bool, optional): A Boolean value that determines the read consistency model during the scan:   If ConsistentRead is false, then the data returned from Scan might not contain the results from other recently completed write operations (PutItem, UpdateItem, or DeleteItem).   If ConsistentRead is true, then all of the write operations that completed before the Scan began are guaranteed to be contained in the Scan response.   The default setting for ConsistentRead is false. The ConsistentRead parameter is not supported on global secondary indexes. If you scan a global secondary index with ConsistentRead set to true, you will receive a ValidationException. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dynamodb.init.scan(ctx, table_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dynamodb.init.scan table_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dynamodb",
        operation="scan",
        op_kwargs={
            "TableName": table_name,
            "IndexName": index_name,
            "AttributesToGet": attributes_to_get,
            "Limit": limit,
            "Select": select,
            "ScanFilter": scan_filter,
            "ConditionalOperator": conditional_operator,
            "ExclusiveStartKey": exclusive_start_key,
            "ReturnConsumedCapacity": return_consumed_capacity,
            "TotalSegments": total_segments,
            "Segment": segment,
            "ProjectionExpression": projection_expression,
            "FilterExpression": filter_expression,
            "ExpressionAttributeNames": expression_attribute_names,
            "ExpressionAttributeValues": expression_attribute_values,
            "ConsistentRead": consistent_read,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub, ctx, virtual_interface_id: str = None, new_bgp_peer: Dict = None
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a BGP peer on the specified virtual interface. You must create a BGP peer for the corresponding address family (IPv4/IPv6) in order to access Amazon Web Services resources that also use that address family. If logical redundancy is not supported by the connection, interconnect, or LAG, the BGP peer cannot be in the same address family as an existing BGP peer on the virtual interface. When creating a IPv6 BGP peer, omit the Amazon address and customer address. IPv6 addresses are automatically assigned from the Amazon pool of IPv6 addresses; you cannot specify custom IPv6 addresses. For a public virtual interface, the Autonomous System Number (ASN) must be private or already on the allow list for the virtual interface.

    Args:
        virtual_interface_id(str, optional): The ID of the virtual interface. Defaults to None.
        new_bgp_peer(Dict, optional): Information about the BGP peer. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.directconnect.bgp.peer.create(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.directconnect.bgp.peer.create
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="directconnect",
        operation="create_bgp_peer",
        op_kwargs={
            "virtualInterfaceId": virtual_interface_id,
            "newBGPPeer": new_bgp_peer,
        },
    )


async def delete(
    hub,
    ctx,
    virtual_interface_id: str = None,
    asn: int = None,
    customer_address: str = None,
    bgp_peer_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the specified BGP peer on the specified virtual interface with the specified customer address and ASN. You cannot delete the last BGP peer from a virtual interface.

    Args:
        virtual_interface_id(str, optional): The ID of the virtual interface. Defaults to None.
        asn(int, optional): The autonomous system (AS) number for Border Gateway Protocol (BGP) configuration. Defaults to None.
        customer_address(str, optional): The IP address assigned to the customer interface. Defaults to None.
        bgp_peer_id(str, optional): The ID of the BGP peer. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.directconnect.bgp.peer.delete(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.directconnect.bgp.peer.delete
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="directconnect",
        operation="delete_bgp_peer",
        op_kwargs={
            "virtualInterfaceId": virtual_interface_id,
            "asn": asn,
            "customerAddress": customer_address,
            "bgpPeerId": bgp_peer_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def describe_all(
    hub,
    ctx,
    local_gateway_route_table_ids: List = None,
    filters: List = None,
    max_results: int = None,
    next_token: str = None,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Describes one or more local gateway route tables. By default, all local gateway route tables are described. Alternatively, you can filter the results.

    Args:
        local_gateway_route_table_ids(List, optional): The IDs of the local gateway route tables. Defaults to None.
        filters(List, optional): One or more filters.    local-gateway-id - The ID of a local gateway.    local-gateway-route-table-id - The ID of a local gateway route table.    outpost-arn - The Amazon Resource Name (ARN) of the Outpost.    state - The state of the local gateway route table.  . Defaults to None.
        max_results(int, optional): The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned nextToken value. Defaults to None.
        next_token(str, optional): The token for the next page of results. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.local_gateway.route.table.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.local_gateway.route.table.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="describe_local_gateway_route_tables",
        op_kwargs={
            "LocalGatewayRouteTableIds": local_gateway_route_table_ids,
            "Filters": filters,
            "MaxResults": max_results,
            "NextToken": next_token,
            "DryRun": dry_run,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(hub, ctx, next_token: str = None, max_results: int = None) -> Dict:
    r"""
    **Autogenerated function**

     Returns a list of the existing Amplify apps.

    Args:
        next_token(str, optional):  A pagination token. If non-null, the pagination token is returned in a result. Pass its value in another request to retrieve more entries. . Defaults to None.
        max_results(int, optional):  The maximum number of records to list in a single response. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.amplify.apps.list(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.amplify.apps.list
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="amplify",
        operation="list_apps",
        op_kwargs={"nextToken": next_token, "maxResults": max_results},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def delete(hub, ctx, access_token: str) -> None:
    r"""
    **Autogenerated function**

    Allows a user to delete himself or herself.

    Args:
        access_token(str): The access token from a request to delete a user.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_idp.user.init.delete(ctx, access_token=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_idp.user.init.delete access_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-idp",
        operation="delete_user",
        op_kwargs={"AccessToken": access_token},
    )


async def get(hub, ctx, access_token: str) -> Dict:
    r"""
    **Autogenerated function**

    Gets the user attributes and metadata for a user.

    Args:
        access_token(str): The access token returned by the server response to get information about the user.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_idp.user.init.get(ctx, access_token=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_idp.user.init.get access_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-idp",
        operation="get_user",
        op_kwargs={"AccessToken": access_token},
    )


async def list_all(
    hub,
    ctx,
    user_pool_id: str,
    attributes_to_get: List = None,
    limit: int = None,
    pagination_token: str = None,
    filter_: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the users in the Amazon Cognito user pool.

    Args:
        user_pool_id(str): The user pool ID for the user pool on which the search should be performed.
        attributes_to_get(List, optional): An array of strings, where each string is the name of a user attribute to be returned for each user in the search results. If the array is null, all attributes are returned. Defaults to None.
        limit(int, optional): Maximum number of users to be returned. Defaults to None.
        pagination_token(str, optional): An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list. Defaults to None.
        filter_(str, optional): A filter string of the form "AttributeName Filter-Type "AttributeValue"". Quotation marks within the filter string must be escaped using the backslash (\) character. For example, "family_name = \"Reddy\"".    AttributeName: The name of the attribute to search for. You can only search for one attribute at a time.    Filter-Type: For an exact match, use =, for example, "given_name = \"Jon\"". For a prefix ("starts with") match, use ^=, for example, "given_name ^= \"Jon\"".     AttributeValue: The attribute value that must be matched for each user.   If the filter string is empty, ListUsers returns all users in the user pool. You can only search for the following standard attributes:    username (case-sensitive)    email     phone_number     name     given_name     family_name     preferred_username     cognito:user_status (called Status in the Console) (case-insensitive)    status (called Enabled in the Console) (case-sensitive)     sub    Custom attributes are not searchable. For more information, see Searching for Users Using the ListUsers API and Examples of Using the ListUsers API in the Amazon Cognito Developer Guide. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_idp.user.init.list_all(ctx, user_pool_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_idp.user.init.list_all user_pool_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-idp",
        operation="list_users",
        op_kwargs={
            "UserPoolId": user_pool_id,
            "AttributesToGet": attributes_to_get,
            "Limit": limit,
            "PaginationToken": pagination_token,
            "Filter": filter_,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe(hub, ctx) -> Dict:
    r"""
    **Autogenerated function**

    Describes the status of the StartReportCreation operation.  You can call this operation only from the organization's management account and from the us-east-1 Region.


    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.resourcegroupstaggingapi.report_creation.describe(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.resourcegroupstaggingapi.report_creation.describe
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="resourcegroupstaggingapi",
        operation="describe_report_creation",
        op_kwargs={},
    )


async def start(hub, ctx, s3_bucket: str) -> Dict:
    r"""
    **Autogenerated function**

    Generates a report that lists all tagged resources in the accounts across your organization and tells whether each resource is compliant with the effective tag policy. Compliance data is refreshed daily. The report is generated asynchronously. The generated report is saved to the following location:  s3://example-bucket/AwsTagPolicies/o-exampleorgid/YYYY-MM-ddTHH:mm:ssZ/report.csv  You can call this operation only from the organization's management account and from the us-east-1 Region.

    Args:
        s3_bucket(str): The name of the Amazon S3 bucket where the report will be stored; for example:  awsexamplebucket  For more information on S3 bucket requirements, including an example bucket policy, see the example S3 bucket policy on this page.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.resourcegroupstaggingapi.report_creation.start(
                ctx, s3_bucket=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.resourcegroupstaggingapi.report_creation.start s3_bucket=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="resourcegroupstaggingapi",
        operation="start_report_creation",
        op_kwargs={"S3Bucket": s3_bucket},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def start(hub, ctx, room_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Initiates the discovery of any smart home appliances associated with the room.

    Args:
        room_arn(str): The room where smart home appliance discovery was initiated.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.alexaforbusiness.smart_home_appliance.discovery.start(
                ctx, room_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.alexaforbusiness.smart_home_appliance.discovery.start room_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="alexaforbusiness",
        operation="start_smart_home_appliance_discovery",
        op_kwargs={"RoomArn": room_arn},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def describe_all(
    hub,
    ctx,
    network_insights_analysis_ids: List = None,
    network_insights_path_id: str = None,
    analysis_start_time: str = None,
    analysis_end_time: str = None,
    filters: List = None,
    max_results: int = None,
    dry_run: bool = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Describes one or more of your network insights analyses.

    Args:
        network_insights_analysis_ids(List, optional): The ID of the network insights analyses. You must specify either analysis IDs or a path ID. Defaults to None.
        network_insights_path_id(str, optional): The ID of the path. You must specify either a path ID or analysis IDs. Defaults to None.
        analysis_start_time(str, optional): The time when the network insights analyses started. Defaults to None.
        analysis_end_time(str, optional): The time when the network insights analyses ended. Defaults to None.
        filters(List, optional): The filters. The following are possible values:   PathFound - A Boolean value that indicates whether a feasible path is found.   Status - The status of the analysis (running | succeeded | failed).  . Defaults to None.
        max_results(int, optional): The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned nextToken value. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        next_token(str, optional): The token for the next page of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.network.insight.analyse.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.network.insight.analyse.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="describe_network_insights_analyses",
        op_kwargs={
            "NetworkInsightsAnalysisIds": network_insights_analysis_ids,
            "NetworkInsightsPathId": network_insights_path_id,
            "AnalysisStartTime": analysis_start_time,
            "AnalysisEndTime": analysis_end_time,
            "Filters": filters,
            "MaxResults": max_results,
            "DryRun": dry_run,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import List


__func_alias__ = {"set_": "set"}


async def set_(hub, ctx, job_flow_ids: List, termination_protected: bool) -> None:
    r"""
    **Autogenerated function**

    SetTerminationProtection locks a cluster (job flow) so the EC2 instances in the cluster cannot be terminated by user intervention, an API call, or in the event of a job-flow error. The cluster still terminates upon successful completion of the job flow. Calling SetTerminationProtection on a cluster is similar to calling the Amazon EC2 DisableAPITermination API on all EC2 instances in a cluster.  SetTerminationProtection is used to prevent accidental termination of a cluster and to ensure that in the event of an error, the instances persist so that you can recover any data stored in their ephemeral instance storage.  To terminate a cluster that has been locked by setting SetTerminationProtection to true, you must first unlock the job flow by a subsequent call to SetTerminationProtection in which you set the value to false.   For more information, seeManaging Cluster Termination in the Amazon EMR Management Guide.

    Args:
        job_flow_ids(List):  A list of strings that uniquely identify the clusters to protect. This identifier is returned by RunJobFlow and can also be obtained from DescribeJobFlows . .
        termination_protected(bool): A Boolean that indicates whether to protect the cluster and prevent the Amazon EC2 instances in the cluster from shutting down due to API calls, user intervention, or job-flow error.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.emr.termination_protection.set(
                ctx, job_flow_ids=value, termination_protected=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.emr.termination_protection.set job_flow_ids=value, termination_protected=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="emr",
        operation="set_termination_protection",
        op_kwargs={
            "JobFlowIds": job_flow_ids,
            "TerminationProtected": termination_protected,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get_all(
    hub,
    ctx,
    prefix_list_id: str,
    dry_run: bool = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Gets information about the resources that are associated with the specified managed prefix list.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        prefix_list_id(str): The ID of the prefix list.
        max_results(int, optional): The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned nextToken value. Defaults to None.
        next_token(str, optional): The token for the next page of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.managed_prefix_list_.association.get_all(
                ctx, prefix_list_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.managed_prefix_list_.association.get_all prefix_list_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="get_managed_prefix_list_associations",
        op_kwargs={
            "DryRun": dry_run,
            "PrefixListId": prefix_list_id,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def batch(hub, ctx, create_delegation_requests: List, assessment_id: str) -> Dict:
    r"""
    **Autogenerated function**

     Create a batch of delegations for a specified assessment in Audit Manager.

    Args:
        create_delegation_requests(List):  The API request to batch create delegations in Audit Manager. .
        assessment_id(str):  The identifier for the specified assessment. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.auditmanager.create_delegation_by_assessment.batch(
                ctx, create_delegation_requests=value, assessment_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.auditmanager.create_delegation_by_assessment.batch create_delegation_requests=value, assessment_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="auditmanager",
        operation="batch_create_delegation_by_assessment",
        op_kwargs={
            "createDelegationRequests": create_delegation_requests,
            "assessmentId": assessment_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def remove(hub, ctx, resource_arn: str, tag_keys: List) -> Dict:
    r"""
    **Autogenerated function**

    Removes metadata tags from an DMS resource, including replication instance, endpoint, security group, and migration task. For more information, see  Tag  data type description.

    Args:
        resource_arn(str): An DMS resource from which you want to remove tag(s). The value for this parameter is an Amazon Resource Name (ARN).
        tag_keys(List): The tag key (name) of the tag to be removed.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dms.tags.from_resource.remove(
                ctx, resource_arn=value, tag_keys=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dms.tags.from_resource.remove resource_arn=value, tag_keys=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dms",
        operation="remove_tags_from_resource",
        op_kwargs={"ResourceArn": resource_arn, "TagKeys": tag_keys},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def cancel(
    hub,
    ctx,
    spot_fleet_request_ids: List,
    terminate_instances: bool,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Cancels the specified Spot Fleet requests. After you cancel a Spot Fleet request, the Spot Fleet launches no new Spot Instances. You must specify whether the Spot Fleet should also terminate its Spot Instances. If you terminate the instances, the Spot Fleet request enters the cancelled_terminating state. Otherwise, the Spot Fleet request enters the cancelled_running state and the instances continue to run until they are interrupted or you terminate them manually.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        spot_fleet_request_ids(List): The IDs of the Spot Fleet requests.
        terminate_instances(bool): Indicates whether to terminate instances for a Spot Fleet request if it is canceled successfully.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.spot.fleet.request.init.cancel(
                ctx, spot_fleet_request_ids=value, terminate_instances=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.spot.fleet.request.init.cancel spot_fleet_request_ids=value, terminate_instances=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="cancel_spot_fleet_requests",
        op_kwargs={
            "DryRun": dry_run,
            "SpotFleetRequestIds": spot_fleet_request_ids,
            "TerminateInstances": terminate_instances,
        },
    )


async def describe_all(
    hub,
    ctx,
    dry_run: bool = None,
    max_results: int = None,
    next_token: str = None,
    spot_fleet_request_ids: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Describes your Spot Fleet requests. Spot Fleet requests are deleted 48 hours after they are canceled and their instances are terminated.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        max_results(int, optional): The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another call with the returned NextToken value. Defaults to None.
        next_token(str, optional): The token for the next set of results. Defaults to None.
        spot_fleet_request_ids(List, optional): The IDs of the Spot Fleet requests. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.spot.fleet.request.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.spot.fleet.request.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="describe_spot_fleet_requests",
        op_kwargs={
            "DryRun": dry_run,
            "MaxResults": max_results,
            "NextToken": next_token,
            "SpotFleetRequestIds": spot_fleet_request_ids,
        },
    )


async def modify(
    hub,
    ctx,
    spot_fleet_request_id: str,
    excess_capacity_termination_policy: str = None,
    launch_template_configs: List = None,
    target_capacity: int = None,
    on_demand_target_capacity: int = None,
    context: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the specified Spot Fleet request. You can only modify a Spot Fleet request of type maintain. While the Spot Fleet request is being modified, it is in the modifying state. To scale up your Spot Fleet, increase its target capacity. The Spot Fleet launches the additional Spot Instances according to the allocation strategy for the Spot Fleet request. If the allocation strategy is lowestPrice, the Spot Fleet launches instances using the Spot Instance pool with the lowest price. If the allocation strategy is diversified, the Spot Fleet distributes the instances across the Spot Instance pools. If the allocation strategy is capacityOptimized, Spot Fleet launches instances from Spot Instance pools with optimal capacity for the number of instances that are launching. To scale down your Spot Fleet, decrease its target capacity. First, the Spot Fleet cancels any open requests that exceed the new target capacity. You can request that the Spot Fleet terminate Spot Instances until the size of the fleet no longer exceeds the new target capacity. If the allocation strategy is lowestPrice, the Spot Fleet terminates the instances with the highest price per unit. If the allocation strategy is capacityOptimized, the Spot Fleet terminates the instances in the Spot Instance pools that have the least available Spot Instance capacity. If the allocation strategy is diversified, the Spot Fleet terminates instances across the Spot Instance pools. Alternatively, you can request that the Spot Fleet keep the fleet at its current size, but not replace any Spot Instances that are interrupted or that you terminate manually. If you are finished with your Spot Fleet for now, but will use it again later, you can set the target capacity to 0.

    Args:
        excess_capacity_termination_policy(str, optional): Indicates whether running Spot Instances should be terminated if the target capacity of the Spot Fleet request is decreased below the current size of the Spot Fleet. Defaults to None.
        launch_template_configs(List, optional): The launch template and overrides. You can only use this parameter if you specified a launch template (LaunchTemplateConfigs) in your Spot Fleet request. If you specified LaunchSpecifications in your Spot Fleet request, then omit this parameter. Defaults to None.
        spot_fleet_request_id(str): The ID of the Spot Fleet request.
        target_capacity(int, optional): The size of the fleet. Defaults to None.
        on_demand_target_capacity(int, optional): The number of On-Demand Instances in the fleet. Defaults to None.
        context(str, optional): Reserved. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.spot.fleet.request.init.modify(
                ctx, spot_fleet_request_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.spot.fleet.request.init.modify spot_fleet_request_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="modify_spot_fleet_request",
        op_kwargs={
            "ExcessCapacityTerminationPolicy": excess_capacity_termination_policy,
            "LaunchTemplateConfigs": launch_template_configs,
            "SpotFleetRequestId": spot_fleet_request_id,
            "TargetCapacity": target_capacity,
            "OnDemandTargetCapacity": on_demand_target_capacity,
            "Context": context,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def associate(hub, ctx, trial_component_name: str, trial_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Associates a trial component with a trial. A trial component can be associated with multiple trials. To disassociate a trial component from a trial, call the DisassociateTrialComponent API.

    Args:
        trial_component_name(str): The name of the component to associated with the trial.
        trial_name(str): The name of the trial to associate with.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.trial.component.associate(
                ctx, trial_component_name=value, trial_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.trial.component.associate trial_component_name=value, trial_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="associate_trial_component",
        op_kwargs={"TrialComponentName": trial_component_name, "TrialName": trial_name},
    )


async def create(
    hub,
    ctx,
    trial_component_name: str,
    display_name: str = None,
    status: Dict = None,
    start_time: str = None,
    end_time: str = None,
    parameters: Dict = None,
    input_artifacts: Dict = None,
    output_artifacts: Dict = None,
    metadata_properties: Dict = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a trial component, which is a stage of a machine learning trial. A trial is composed of one or more trial components. A trial component can be used in multiple trials. Trial components include pre-processing jobs, training jobs, and batch transform jobs. When you use SageMaker Studio or the SageMaker Python SDK, all experiments, trials, and trial components are automatically tracked, logged, and indexed. When you use the Amazon Web Services SDK for Python (Boto), you must use the logging APIs provided by the SDK. You can add tags to a trial component and then use the Search API to search for the tags.

    Args:
        trial_component_name(str): The name of the component. The name must be unique in your Amazon Web Services account and is not case-sensitive.
        display_name(str, optional): The name of the component as displayed. The name doesn't need to be unique. If DisplayName isn't specified, TrialComponentName is displayed. Defaults to None.
        status(Dict, optional): The status of the component. States include:   InProgress   Completed   Failed  . Defaults to None.
        start_time(str, optional): When the component started. Defaults to None.
        end_time(str, optional): When the component ended. Defaults to None.
        parameters(Dict, optional): The hyperparameters for the component. Defaults to None.
        input_artifacts(Dict, optional): The input artifacts for the component. Examples of input artifacts are datasets, algorithms, hyperparameters, source code, and instance types. Defaults to None.
        output_artifacts(Dict, optional): The output artifacts for the component. Examples of output artifacts are metrics, snapshots, logs, and images. Defaults to None.
        metadata_properties(Dict, optional): Metadata properties of the tracking entity, trial, or trial component. Defaults to None.
        tags(List, optional): A list of tags to associate with the component. You can use Search API to search on the tags. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.trial.component.create(
                ctx, trial_component_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.trial.component.create trial_component_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="create_trial_component",
        op_kwargs={
            "TrialComponentName": trial_component_name,
            "DisplayName": display_name,
            "Status": status,
            "StartTime": start_time,
            "EndTime": end_time,
            "Parameters": parameters,
            "InputArtifacts": input_artifacts,
            "OutputArtifacts": output_artifacts,
            "MetadataProperties": metadata_properties,
            "Tags": tags,
        },
    )


async def delete(hub, ctx, trial_component_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the specified trial component. A trial component must be disassociated from all trials before the trial component can be deleted. To disassociate a trial component from a trial, call the DisassociateTrialComponent API.

    Args:
        trial_component_name(str): The name of the component to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.trial.component.delete(
                ctx, trial_component_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.trial.component.delete trial_component_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="delete_trial_component",
        op_kwargs={"TrialComponentName": trial_component_name},
    )


async def describe(hub, ctx, trial_component_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Provides a list of a trials component's properties.

    Args:
        trial_component_name(str): The name of the trial component to describe.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.trial.component.describe(
                ctx, trial_component_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.trial.component.describe trial_component_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="describe_trial_component",
        op_kwargs={"TrialComponentName": trial_component_name},
    )


async def disassociate(hub, ctx, trial_component_name: str, trial_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Disassociates a trial component from a trial. This doesn't effect other trials the component is associated with. Before you can delete a component, you must disassociate the component from all trials it is associated with. To associate a trial component with a trial, call the AssociateTrialComponent API. To get a list of the trials a component is associated with, use the Search API. Specify ExperimentTrialComponent for the Resource parameter. The list appears in the response under Results.TrialComponent.Parents.

    Args:
        trial_component_name(str): The name of the component to disassociate from the trial.
        trial_name(str): The name of the trial to disassociate from.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.trial.component.disassociate(
                ctx, trial_component_name=value, trial_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.trial.component.disassociate trial_component_name=value, trial_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="disassociate_trial_component",
        op_kwargs={"TrialComponentName": trial_component_name, "TrialName": trial_name},
    )


async def list_all(
    hub,
    ctx,
    experiment_name: str = None,
    trial_name: str = None,
    source_arn: str = None,
    created_after: str = None,
    created_before: str = None,
    sort_by: str = None,
    sort_order: str = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the trial components in your account. You can sort the list by trial component name or creation time. You can filter the list to show only components that were created in a specific time range. You can also filter on one of the following:    ExperimentName     SourceArn     TrialName

    Args:
        experiment_name(str, optional): A filter that returns only components that are part of the specified experiment. If you specify ExperimentName, you can't filter by SourceArn or TrialName. Defaults to None.
        trial_name(str, optional): A filter that returns only components that are part of the specified trial. If you specify TrialName, you can't filter by ExperimentName or SourceArn. Defaults to None.
        source_arn(str, optional): A filter that returns only components that have the specified source Amazon Resource Name (ARN). If you specify SourceArn, you can't filter by ExperimentName or TrialName. Defaults to None.
        created_after(str, optional): A filter that returns only components created after the specified time. Defaults to None.
        created_before(str, optional): A filter that returns only components created before the specified time. Defaults to None.
        sort_by(str, optional): The property used to sort results. The default value is CreationTime. Defaults to None.
        sort_order(str, optional): The sort order. The default value is Descending. Defaults to None.
        max_results(int, optional): The maximum number of components to return in the response. The default value is 10. Defaults to None.
        next_token(str, optional): If the previous call to ListTrialComponents didn't return the full set of components, the call returns a token for getting the next set of components. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.trial.component.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.trial.component.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="list_trial_components",
        op_kwargs={
            "ExperimentName": experiment_name,
            "TrialName": trial_name,
            "SourceArn": source_arn,
            "CreatedAfter": created_after,
            "CreatedBefore": created_before,
            "SortBy": sort_by,
            "SortOrder": sort_order,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )


async def update(
    hub,
    ctx,
    trial_component_name: str,
    display_name: str = None,
    status: Dict = None,
    start_time: str = None,
    end_time: str = None,
    parameters: Dict = None,
    parameters_to_remove: List = None,
    input_artifacts: Dict = None,
    input_artifacts_to_remove: List = None,
    output_artifacts: Dict = None,
    output_artifacts_to_remove: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates one or more properties of a trial component.

    Args:
        trial_component_name(str): The name of the component to update.
        display_name(str, optional): The name of the component as displayed. The name doesn't need to be unique. If DisplayName isn't specified, TrialComponentName is displayed. Defaults to None.
        status(Dict, optional): The new status of the component. Defaults to None.
        start_time(str, optional): When the component started. Defaults to None.
        end_time(str, optional): When the component ended. Defaults to None.
        parameters(Dict, optional): Replaces all of the component's hyperparameters with the specified hyperparameters. Defaults to None.
        parameters_to_remove(List, optional): The hyperparameters to remove from the component. Defaults to None.
        input_artifacts(Dict, optional): Replaces all of the component's input artifacts with the specified artifacts. Defaults to None.
        input_artifacts_to_remove(List, optional): The input artifacts to remove from the component. Defaults to None.
        output_artifacts(Dict, optional): Replaces all of the component's output artifacts with the specified artifacts. Defaults to None.
        output_artifacts_to_remove(List, optional): The output artifacts to remove from the component. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.trial.component.update(
                ctx, trial_component_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.trial.component.update trial_component_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="update_trial_component",
        op_kwargs={
            "TrialComponentName": trial_component_name,
            "DisplayName": display_name,
            "Status": status,
            "StartTime": start_time,
            "EndTime": end_time,
            "Parameters": parameters,
            "ParametersToRemove": parameters_to_remove,
            "InputArtifacts": input_artifacts,
            "InputArtifactsToRemove": input_artifacts_to_remove,
            "OutputArtifacts": output_artifacts,
            "OutputArtifactsToRemove": output_artifacts_to_remove,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""


async def disable(hub, ctx, key_id: str) -> None:
    r"""
    **Autogenerated function**

    Disables automatic rotation of the key material for the specified symmetric customer master key (CMK).  You cannot enable automatic rotation of asymmetric CMKs, CMKs with imported key material, or CMKs in a custom key store. To enable or disable automatic rotation of a set of related multi-Region keys, set the property on the primary key.  The CMK that you use for this operation must be in a compatible key state. For details, see Key state: Effect on your CMK in the AWS Key Management Service Developer Guide.  Cross-account use: No. You cannot perform this operation on a CMK in a different AWS account.  Required permissions: kms:DisableKeyRotation (key policy)  Related operations:     EnableKeyRotation     GetKeyRotationStatus

    Args:
        key_id(str): Identifies a symmetric customer master key (CMK). You cannot enable or disable automatic rotation of asymmetric CMKs, CMKs with imported key material, or CMKs in a custom key store. Specify the key ID or key ARN of the CMK. For example:   Key ID: 1234abcd-12ab-34cd-56ef-1234567890ab    Key ARN: arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab    To get the key ID and key ARN for a CMK, use ListKeys or DescribeKey.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.kms.key.rotation.init.disable(ctx, key_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.kms.key.rotation.init.disable key_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="kms",
        operation="disable_key_rotation",
        op_kwargs={"KeyId": key_id},
    )


async def enable(hub, ctx, key_id: str) -> None:
    r"""
    **Autogenerated function**

    Enables automatic rotation of the key material for the specified symmetric customer master key (CMK). You cannot enable automatic rotation of asymmetric CMKs, CMKs with imported key material, or CMKs in a custom key store. To enable or disable automatic rotation of a set of related multi-Region keys, set the property on the primary key. The CMK that you use for this operation must be in a compatible key state. For details, see Key state: Effect on your CMK in the AWS Key Management Service Developer Guide.  Cross-account use: No. You cannot perform this operation on a CMK in a different AWS account.  Required permissions: kms:EnableKeyRotation (key policy)  Related operations:     DisableKeyRotation     GetKeyRotationStatus

    Args:
        key_id(str): Identifies a symmetric customer master key (CMK). You cannot enable automatic rotation of asymmetric CMKs, CMKs with imported key material, or CMKs in a custom key store. To enable or disable automatic rotation of a set of related multi-Region keys, set the property on the primary key. Specify the key ID or key ARN of the CMK. For example:   Key ID: 1234abcd-12ab-34cd-56ef-1234567890ab    Key ARN: arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab    To get the key ID and key ARN for a CMK, use ListKeys or DescribeKey.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.kms.key.rotation.init.enable(ctx, key_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.kms.key.rotation.init.enable key_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="kms",
        operation="enable_key_rotation",
        op_kwargs={"KeyId": key_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"set_": "set"}


async def set_(
    hub,
    ctx,
    identity: str,
    mail_from_domain: str = None,
    behavior_on_mx_failure: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Enables or disables the custom MAIL FROM domain setup for a verified identity (an email address or a domain).  To send emails using the specified MAIL FROM domain, you must add an MX record to your MAIL FROM domain's DNS settings. If you want your emails to pass Sender Policy Framework (SPF) checks, you must also add or update an SPF record. For more information, see the Amazon SES Developer Guide.  You can execute this operation no more than once per second.

    Args:
        identity(str): The verified identity for which you want to enable or disable the specified custom MAIL FROM domain.
        mail_from_domain(str, optional): The custom MAIL FROM domain that you want the verified identity to use. The MAIL FROM domain must 1) be a subdomain of the verified identity, 2) not be used in a "From" address if the MAIL FROM domain is the destination of email feedback forwarding (for more information, see the Amazon SES Developer Guide), and 3) not be used to receive emails. A value of null disables the custom MAIL FROM setting for the identity. Defaults to None.
        behavior_on_mx_failure(str, optional): The action that you want Amazon SES to take if it cannot successfully read the required MX record when you send an email. If you choose UseDefaultValue, Amazon SES will use amazonses.com (or a subdomain of that) as the MAIL FROM domain. If you choose RejectMessage, Amazon SES will return a MailFromDomainNotVerified error and not send the email. The action specified in BehaviorOnMXFailure is taken when the custom MAIL FROM domain setup is in the Pending, Failed, and TemporaryFailure states. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ses.identity.mail_from_domain.init.set(ctx, identity=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ses.identity.mail_from_domain.init.set identity=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ses",
        operation="set_identity_mail_from_domain",
        op_kwargs={
            "Identity": identity,
            "MailFromDomain": mail_from_domain,
            "BehaviorOnMXFailure": behavior_on_mx_failure,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, resource_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Permanently deletes an IAM policy from the specified rule group. You must be the owner of the rule group to perform this operation.

    Args:
        resource_arn(str): The Amazon Resource Name (ARN) of the rule group from which you want to delete the policy. You must be the owner of the rule group to perform this operation.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.wafv2.permission_policy.delete(ctx, resource_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.wafv2.permission_policy.delete resource_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="wafv2",
        operation="delete_permission_policy",
        op_kwargs={"ResourceArn": resource_arn},
    )


async def get(hub, ctx, resource_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns the IAM policy that is attached to the specified rule group. You must be the owner of the rule group to perform this operation.

    Args:
        resource_arn(str): The Amazon Resource Name (ARN) of the rule group for which you want to get the policy.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.wafv2.permission_policy.get(ctx, resource_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.wafv2.permission_policy.get resource_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="wafv2",
        operation="get_permission_policy",
        op_kwargs={"ResourceArn": resource_arn},
    )


async def put(hub, ctx, resource_arn: str, policy: str) -> Dict:
    r"""
    **Autogenerated function**

    Attaches an IAM policy to the specified resource. Use this to share a rule group across accounts. You must be the owner of the rule group to perform this operation. This action is subject to the following restrictions:   You can attach only one policy with each PutPermissionPolicy request.   The ARN in the request must be a valid WAF RuleGroup ARN and the rule group must exist in the same Region.   The user making the request must be the owner of the rule group.

    Args:
        resource_arn(str): The Amazon Resource Name (ARN) of the RuleGroup to which you want to attach the policy.
        policy(str): The policy to attach to the specified rule group.  The policy specifications must conform to the following:   The policy must be composed using IAM Policy version 2012-10-17 or version 2015-01-01.   The policy must include specifications for Effect, Action, and Principal.    Effect must specify Allow.    Action must specify wafv2:CreateWebACL, wafv2:UpdateWebACL, and wafv2:PutFirewallManagerRuleGroups. WAF rejects any extra actions or wildcard actions in the policy.   The policy must not include a Resource parameter.   For more information, see IAM Policies. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.wafv2.permission_policy.put(
                ctx, resource_arn=value, policy=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.wafv2.permission_policy.put resource_arn=value, policy=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="wafv2",
        operation="put_permission_policy",
        op_kwargs={"ResourceArn": resource_arn, "Policy": policy},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    channel_name: str,
    program_name: str,
    schedule_configuration: Dict,
    source_location_name: str,
    vod_source_name: str,
    ad_breaks: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a program.

    Args:
        ad_breaks(List, optional): The ad break configuration settings. Defaults to None.
        channel_name(str): The identifier for the channel you are working on.
        program_name(str): The identifier for the program you are working on.
        schedule_configuration(Dict): The schedule configuration settings.
        source_location_name(str): The name of the source location.
        vod_source_name(str): The name that's used to refer to a VOD source.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mediatailor.program.create(
                ctx,
                channel_name=value,
                program_name=value,
                schedule_configuration=value,
                source_location_name=value,
                vod_source_name=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mediatailor.program.create channel_name=value, program_name=value, schedule_configuration=value, source_location_name=value, vod_source_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mediatailor",
        operation="create_program",
        op_kwargs={
            "AdBreaks": ad_breaks,
            "ChannelName": channel_name,
            "ProgramName": program_name,
            "ScheduleConfiguration": schedule_configuration,
            "SourceLocationName": source_location_name,
            "VodSourceName": vod_source_name,
        },
    )


async def delete(hub, ctx, channel_name: str, program_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a specific program on a specific channel.

    Args:
        channel_name(str): The identifier for the channel you are working on.
        program_name(str): The identifier for the program you are working on.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mediatailor.program.delete(
                ctx, channel_name=value, program_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mediatailor.program.delete channel_name=value, program_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mediatailor",
        operation="delete_program",
        op_kwargs={"ChannelName": channel_name, "ProgramName": program_name},
    )


async def describe(hub, ctx, channel_name: str, program_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the properties of the requested program.

    Args:
        channel_name(str): The identifier for the channel you are working on.
        program_name(str): The identifier for the program you are working on.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mediatailor.program.describe(
                ctx, channel_name=value, program_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mediatailor.program.describe channel_name=value, program_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mediatailor",
        operation="describe_program",
        op_kwargs={"ChannelName": channel_name, "ProgramName": program_name},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def delete_multiple(
    hub,
    ctx,
    domain_name: str,
    item_name: str,
    attributes: List = None,
    expected: Dict = None,
) -> None:
    r"""
    **Autogenerated function**

     Deletes one or more attributes associated with an item. If all attributes of the item are deleted, the item is deleted.   DeleteAttributes is an idempotent operation; running it multiple times on the same item or attribute does not result in an error response.   Because Amazon SimpleDB makes multiple copies of item data and uses an eventual consistency update model, performing a GetAttributes or Select operation (read) immediately after a DeleteAttributes or PutAttributes operation (write) might not return updated item data.

    Args:
        domain_name(str): The name of the domain in which to perform the operation.
        item_name(str): The name of the item. Similar to rows on a spreadsheet, items represent individual objects that contain one or more value-attribute pairs.
        attributes(List, optional): A list of Attributes. Similar to columns on a spreadsheet, attributes represent categories of data that can be assigned to items. Defaults to None.
        expected(Dict, optional): The update condition which, if specified, determines whether the specified attributes will be deleted or not. The update condition must be satisfied in order for this request to be processed and the attributes to be deleted. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sdb.attribute.delete_multiple(
                ctx, domain_name=value, item_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sdb.attribute.delete_multiple domain_name=value, item_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sdb",
        operation="delete_attributes",
        op_kwargs={
            "DomainName": domain_name,
            "ItemName": item_name,
            "Attributes": attributes,
            "Expected": expected,
        },
    )


async def get_all(
    hub,
    ctx,
    domain_name: str,
    item_name: str,
    attribute_names: List = None,
    consistent_read: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Returns all of the attributes associated with the specified item. Optionally, the attributes returned can be limited to one or more attributes by specifying an attribute name parameter.   If the item does not exist on the replica that was accessed for this operation, an empty set is returned. The system does not return an error as it cannot guarantee the item does not exist on other replicas.

    Args:
        domain_name(str): The name of the domain in which to perform the operation.
        item_name(str): The name of the item.
        attribute_names(List, optional): The names of the attributes. Defaults to None.
        consistent_read(bool, optional): Determines whether or not strong consistency should be enforced when data is read from SimpleDB. If true, any data previously written to SimpleDB will be returned. Otherwise, results will be consistent eventually, and the client may not see data that was written immediately before your read. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sdb.attribute.get_all(ctx, domain_name=value, item_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sdb.attribute.get_all domain_name=value, item_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sdb",
        operation="get_attributes",
        op_kwargs={
            "DomainName": domain_name,
            "ItemName": item_name,
            "AttributeNames": attribute_names,
            "ConsistentRead": consistent_read,
        },
    )


async def put_multiple(
    hub, ctx, domain_name: str, item_name: str, attributes: List, expected: Dict = None
) -> None:
    r"""
    **Autogenerated function**

     The PutAttributes operation creates or replaces attributes in an item. The client may specify new attributes using a combination of the Attribute.X.Name and Attribute.X.Value parameters. The client specifies the first attribute by the parameters Attribute.0.Name and Attribute.0.Value, the second attribute by the parameters Attribute.1.Name and Attribute.1.Value, and so on.   Attributes are uniquely identified in an item by their name/value combination. For example, a single item can have the attributes { "first_name", "first_value" } and { "first_name", second_value" }. However, it cannot have two attribute instances where both the Attribute.X.Name and Attribute.X.Value are the same.   Optionally, the requestor can supply the Replace parameter for each individual attribute. Setting this value to true causes the new attribute value to replace the existing attribute value(s). For example, if an item has the attributes { 'a', '1' }, { 'b', '2'} and { 'b', '3' } and the requestor calls PutAttributes using the attributes { 'b', '4' } with the Replace parameter set to true, the final attributes of the item are changed to { 'a', '1' } and { 'b', '4' }, which replaces the previous values of the 'b' attribute with the new value.   You cannot specify an empty string as an attribute name.   Because Amazon SimpleDB makes multiple copies of client data and uses an eventual consistency update model, an immediate GetAttributes or Select operation (read) immediately after a PutAttributes or DeleteAttributes operation (write) might not return the updated data.   The following limitations are enforced for this operation:  256 total attribute name-value pairs per item One billion attributes per domain 10 GB of total user data storage per domain

    Args:
        domain_name(str): The name of the domain in which to perform the operation.
        item_name(str): The name of the item.
        attributes(List): The list of attributes.
        expected(Dict, optional): The update condition which, if specified, determines whether the specified attributes will be updated or not. The update condition must be satisfied in order for this request to be processed and the attributes to be updated. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sdb.attribute.put_multiple(
                ctx, domain_name=value, item_name=value, attributes=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sdb.attribute.put_multiple domain_name=value, item_name=value, attributes=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sdb",
        operation="put_attributes",
        op_kwargs={
            "DomainName": domain_name,
            "ItemName": item_name,
            "Attributes": attributes,
            "Expected": expected,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub, ctx, app_id: str, branch_name: str, description: str = None
) -> Dict:
    r"""
    **Autogenerated function**

     Creates a new webhook on an Amplify app.

    Args:
        app_id(str):  The unique ID for an Amplify app. .
        branch_name(str):  The name for a branch that is part of an Amplify app. .
        description(str, optional):  The description for a webhook. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.amplify.webhook.create(ctx, app_id=value, branch_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.amplify.webhook.create app_id=value, branch_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="amplify",
        operation="create_webhook",
        op_kwargs={
            "appId": app_id,
            "branchName": branch_name,
            "description": description,
        },
    )


async def delete(hub, ctx, webhook_id: str) -> Dict:
    r"""
    **Autogenerated function**

     Deletes a webhook.

    Args:
        webhook_id(str):  The unique ID for a webhook. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.amplify.webhook.delete(ctx, webhook_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.amplify.webhook.delete webhook_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="amplify",
        operation="delete_webhook",
        op_kwargs={"webhookId": webhook_id},
    )


async def get(hub, ctx, webhook_id: str) -> Dict:
    r"""
    **Autogenerated function**

     Returns the webhook information that corresponds to a specified webhook ID.

    Args:
        webhook_id(str):  The unique ID for a webhook. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.amplify.webhook.get(ctx, webhook_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.amplify.webhook.get webhook_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="amplify",
        operation="get_webhook",
        op_kwargs={"webhookId": webhook_id},
    )


async def list_all(
    hub, ctx, app_id: str, next_token: str = None, max_results: int = None
) -> Dict:
    r"""
    **Autogenerated function**

     Returns a list of webhooks for an Amplify app.

    Args:
        app_id(str):  The unique ID for an Amplify app. .
        next_token(str, optional):  A pagination token. Set to null to start listing webhooks from the start. If non-null,the pagination token is returned in a result. Pass its value in here to list more webhooks. . Defaults to None.
        max_results(int, optional):  The maximum number of records to list in a single response. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.amplify.webhook.list_all(ctx, app_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.amplify.webhook.list_all app_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="amplify",
        operation="list_webhooks",
        op_kwargs={"appId": app_id, "nextToken": next_token, "maxResults": max_results},
    )


async def update(
    hub, ctx, webhook_id: str, branch_name: str = None, description: str = None
) -> Dict:
    r"""
    **Autogenerated function**

     Updates a webhook.

    Args:
        webhook_id(str):  The unique ID for a webhook. .
        branch_name(str, optional):  The name for a branch that is part of an Amplify app. . Defaults to None.
        description(str, optional):  The description for a webhook. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.amplify.webhook.update(ctx, webhook_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.amplify.webhook.update webhook_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="amplify",
        operation="update_webhook",
        op_kwargs={
            "webhookId": webhook_id,
            "branchName": branch_name,
            "description": description,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, instance_family: str, dry_run: bool = None) -> Dict:
    r"""
    **Autogenerated function**

    Describes the default credit option for CPU usage of a burstable performance instance family. For more information, see Burstable performance instances in the Amazon EC2 User Guide.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        instance_family(str): The instance family.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.default.credit_specification.get(ctx, instance_family=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.default.credit_specification.get instance_family=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="get_default_credit_specification",
        op_kwargs={"DryRun": dry_run, "InstanceFamily": instance_family},
    )


async def modify(
    hub, ctx, instance_family: str, cpu_credits: str, dry_run: bool = None
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the default credit option for CPU usage of burstable performance instances. The default credit option is set at the account level per Amazon Web Services Region, and is specified per instance family. All new burstable performance instances in the account launch using the default credit option.  ModifyDefaultCreditSpecification is an asynchronous operation, which works at an Amazon Web Services Region level and modifies the credit option for each Availability Zone. All zones in a Region are updated within five minutes. But if instances are launched during this operation, they might not get the new credit option until the zone is updated. To verify whether the update has occurred, you can call GetDefaultCreditSpecification and check DefaultCreditSpecification for updates. For more information, see Burstable performance instances in the Amazon EC2 User Guide.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        instance_family(str): The instance family.
        cpu_credits(str): The credit option for CPU usage of the instance family. Valid Values: standard | unlimited .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.default.credit_specification.modify(
                ctx, instance_family=value, cpu_credits=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.default.credit_specification.modify instance_family=value, cpu_credits=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="modify_default_credit_specification",
        op_kwargs={
            "DryRun": dry_run,
            "InstanceFamily": instance_family,
            "CpuCredits": cpu_credits,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    iam_role: str,
    description: str = None,
    default_instance_name: str = None,
    registration_limit: int = None,
    expiration_date: str = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Generates an activation code and activation ID you can use to register your on-premises server or virtual machine (VM) with Amazon Web Services Systems Manager. Registering these machines with Systems Manager makes it possible to manage them using Systems Manager capabilities. You use the activation code and ID when installing SSM Agent on machines in your hybrid environment. For more information about requirements for managing on-premises instances and VMs using Systems Manager, see Setting up Amazon Web Services Systems Manager for hybrid environments in the Amazon Web Services Systems Manager User Guide.   On-premises servers or VMs that are registered with Systems Manager and Amazon Elastic Compute Cloud (Amazon EC2) instances that you manage with Systems Manager are all called managed instances.

    Args:
        description(str, optional): A user-defined description of the resource that you want to register with Systems Manager.   Don't enter personally identifiable information in this field. . Defaults to None.
        default_instance_name(str, optional): The name of the registered, managed instance as it will appear in the Amazon Web Services Systems Manager console or when you use the Amazon Web Services command line tools to list Systems Manager resources.  Don't enter personally identifiable information in this field. . Defaults to None.
        iam_role(str): The Identity and Access Management (IAM) role that you want to assign to the managed instance. This IAMrole must provide AssumeRole permissions for the Amazon Web Services Systems Manager service principal ssm.amazonaws.com. For more information, see Create an IAM service role for a hybrid environment in the Amazon Web Services Systems Manager User Guide.
        registration_limit(int, optional): Specify the maximum number of managed instances you want to register. The default value is 1. Defaults to None.
        expiration_date(str, optional): The date by which this activation request should expire, in timestamp format, such as "2021-07-07T00:00:00". You can specify a date up to 30 days in advance. If you don't provide an expiration date, the activation code expires in 24 hours. Defaults to None.
        tags(List, optional): Optional metadata that you assign to a resource. Tags enable you to categorize a resource in different ways, such as by purpose, owner, or environment. For example, you might want to tag an activation to identify which servers or virtual machines (VMs) in your on-premises environment you intend to activate. In this case, you could specify the following key-value pairs:    Key=OS,Value=Windows     Key=Environment,Value=Production     When you install SSM Agent on your on-premises servers and VMs, you specify an activation ID and code. When you specify the activation ID and code, tags assigned to the activation are automatically applied to the on-premises servers or VMs.  You can't add tags to or delete tags from an existing activation. You can tag your on-premises servers and VMs after they connect to Systems Manager for the first time and are assigned a managed instance ID. This means they are listed in the Amazon Web Services Systems Manager console with an ID that is prefixed with "mi-". For information about how to add tags to your managed instances, see AddTagsToResource. For information about how to remove tags from your managed instances, see RemoveTagsFromResource. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.activation.create(ctx, iam_role=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.activation.create iam_role=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="create_activation",
        op_kwargs={
            "Description": description,
            "DefaultInstanceName": default_instance_name,
            "IamRole": iam_role,
            "RegistrationLimit": registration_limit,
            "ExpirationDate": expiration_date,
            "Tags": tags,
        },
    )


async def delete(hub, ctx, activation_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes an activation. You aren't required to delete an activation. If you delete an activation, you can no longer use it to register additional managed instances. Deleting an activation doesn't de-register managed instances. You must manually de-register managed instances.

    Args:
        activation_id(str): The ID of the activation that you want to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.activation.delete(ctx, activation_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.activation.delete activation_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="delete_activation",
        op_kwargs={"ActivationId": activation_id},
    )


async def describe_all(
    hub, ctx, filters: List = None, max_results: int = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Describes details about the activation, such as the date and time the activation was created, its expiration date, the Identity and Access Management (IAM) role assigned to the instances in the activation, and the number of instances registered by using this activation.

    Args:
        filters(List, optional): A filter to view information about your activations. Defaults to None.
        max_results(int, optional): The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results. Defaults to None.
        next_token(str, optional): A token to start the list. Use this token to get the next set of results. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.activation.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.activation.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="describe_activations",
        op_kwargs={
            "Filters": filters,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get_all(hub, ctx, catalog_id: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the security configuration for a specified catalog.

    Args:
        catalog_id(str, optional): The ID of the Data Catalog to retrieve the security configuration for. If none is provided, the Amazon Web Services account ID is used by default. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.data.catalog_encryption_setting.get_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.data.catalog_encryption_setting.get_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="get_data_catalog_encryption_settings",
        op_kwargs={"CatalogId": catalog_id},
    )


async def put_multiple(
    hub, ctx, data_catalog_encryption_settings: Dict, catalog_id: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Sets the security configuration for a specified catalog. After the configuration has been set, the specified encryption is applied to every catalog write thereafter.

    Args:
        catalog_id(str, optional): The ID of the Data Catalog to set the security configuration for. If none is provided, the Amazon Web Services account ID is used by default. Defaults to None.
        data_catalog_encryption_settings(Dict): The security configuration to set.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.data.catalog_encryption_setting.put_multiple(
                ctx, data_catalog_encryption_settings=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.data.catalog_encryption_setting.put_multiple data_catalog_encryption_settings=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="put_data_catalog_encryption_settings",
        op_kwargs={
            "CatalogId": catalog_id,
            "DataCatalogEncryptionSettings": data_catalog_encryption_settings,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def get(hub, ctx, next_token: str = None, max_results: int = None) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves all current job definitions.

    Args:
        next_token(str, optional): A continuation token, if this is a continuation call. Defaults to None.
        max_results(int, optional): The maximum size of the response. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.jobs.get(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.jobs.get
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="get_jobs",
        op_kwargs={"NextToken": next_token, "MaxResults": max_results},
    )


async def list_(
    hub, ctx, next_token: str = None, max_results: int = None, tags: Dict = None
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the names of all job resources in this Amazon Web Services account, or the resources with the specified tag. This operation allows you to see which resources are available in your account, and their names. This operation takes the optional Tags field, which you can use as a filter on the response so that tagged resources can be retrieved as a group. If you choose to use tags filtering, only resources with the tag are retrieved.

    Args:
        next_token(str, optional): A continuation token, if this is a continuation request. Defaults to None.
        max_results(int, optional): The maximum size of a list to return. Defaults to None.
        tags(Dict, optional): Specifies to return only these tagged resources. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.jobs.list(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.jobs.list
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="list_jobs",
        op_kwargs={"NextToken": next_token, "MaxResults": max_results, "Tags": tags},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, backup_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a specified AWS CloudHSM backup. A backup can be restored up to 7 days after the DeleteBackup request is made. For more information on restoring a backup, see RestoreBackup.

    Args:
        backup_id(str): The ID of the backup to be deleted. To find the ID of a backup, use the DescribeBackups operation.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudhsmv2.backup.init.delete(ctx, backup_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudhsmv2.backup.init.delete backup_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudhsmv2",
        operation="delete_backup",
        op_kwargs={"BackupId": backup_id},
    )


async def describe_all(
    hub,
    ctx,
    next_token: str = None,
    max_results: int = None,
    filters: Dict = None,
    sort_ascending: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Gets information about backups of AWS CloudHSM clusters. This is a paginated operation, which means that each response might contain only a subset of all the backups. When the response contains only a subset of backups, it includes a NextToken value. Use this value in a subsequent DescribeBackups request to get more backups. When you receive a response with no NextToken (or an empty or null value), that means there are no more backups to get.

    Args:
        next_token(str, optional): The NextToken value that you received in the previous response. Use this value to get more backups. Defaults to None.
        max_results(int, optional): The maximum number of backups to return in the response. When there are more backups than the number you specify, the response contains a NextToken value. Defaults to None.
        filters(Dict, optional): One or more filters to limit the items returned in the response. Use the backupIds filter to return only the specified backups. Specify backups by their backup identifier (ID). Use the sourceBackupIds filter to return only the backups created from a source backup. The sourceBackupID of a source backup is returned by the CopyBackupToRegion operation. Use the clusterIds filter to return only the backups for the specified clusters. Specify clusters by their cluster identifier (ID). Use the states filter to return only backups that match the specified state. Use the neverExpires filter to return backups filtered by the value in the neverExpires parameter. True returns all backups exempt from the backup retention policy. False returns all backups with a backup retention policy defined at the cluster. Defaults to None.
        sort_ascending(bool, optional): Designates whether or not to sort the return backups by ascending chronological order of generation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudhsmv2.backup.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudhsmv2.backup.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudhsmv2",
        operation="describe_backups",
        op_kwargs={
            "NextToken": next_token,
            "MaxResults": max_results,
            "Filters": filters,
            "SortAscending": sort_ascending,
        },
    )


async def restore(hub, ctx, backup_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Restores a specified AWS CloudHSM backup that is in the PENDING_DELETION state. For mor information on deleting a backup, see DeleteBackup.

    Args:
        backup_id(str): The ID of the backup to be restored. To find the ID of a backup, use the DescribeBackups operation.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudhsmv2.backup.init.restore(ctx, backup_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudhsmv2.backup.init.restore backup_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudhsmv2",
        operation="restore_backup",
        op_kwargs={"BackupId": backup_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def add(hub, ctx, resource_arn: str, tags: List) -> Dict:
    r"""
    **Autogenerated function**

    Adds metadata tags to an DMS resource, including replication instance, endpoint, security group, and migration task. These tags can also be used with cost allocation reporting to track cost associated with DMS resources, or used in a Condition statement in an IAM policy for DMS. For more information, see  Tag  data type description.

    Args:
        resource_arn(str): Identifies the DMS resource to which tags should be added. The value for this parameter is an Amazon Resource Name (ARN). For DMS, you can tag a replication instance, an endpoint, or a replication task.
        tags(List): One or more tags to be assigned to the resource.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dms.tags.to_resource.add(ctx, resource_arn=value, tags=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dms.tags.to_resource.add resource_arn=value, tags=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dms",
        operation="add_tags_to_resource",
        op_kwargs={"ResourceArn": resource_arn, "Tags": tags},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    name: str,
    auth_mode: str,
    vpc_id: str,
    subnet_ids: List,
    service_role: str,
    user_role: str,
    workspace_security_group_id: str,
    engine_security_group_id: str,
    default_s3_location: str,
    description: str = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a new Amazon EMR Studio.

    Args:
        name(str): A descriptive name for the Amazon EMR Studio.
        description(str, optional): A detailed description of the Amazon EMR Studio. Defaults to None.
        auth_mode(str): Specifies whether the Studio authenticates users using single sign-on (SSO) or IAM. Amazon EMR Studio currently only supports SSO authentication.
        vpc_id(str): The ID of the Amazon Virtual Private Cloud (Amazon VPC) to associate with the Studio.
        subnet_ids(List): A list of subnet IDs to associate with the Amazon EMR Studio. A Studio can have a maximum of 5 subnets. The subnets must belong to the VPC specified by VpcId. Studio users can create a Workspace in any of the specified subnets.
        service_role(str): The IAM role that will be assumed by the Amazon EMR Studio. The service role provides a way for Amazon EMR Studio to interoperate with other Amazon Web Services services.
        user_role(str): The IAM user role that will be assumed by users and groups logged in to an Amazon EMR Studio. The permissions attached to this IAM role can be scoped down for each user or group using session policies.
        workspace_security_group_id(str): The ID of the Amazon EMR Studio Workspace security group. The Workspace security group allows outbound network traffic to resources in the Engine security group, and it must be in the same VPC specified by VpcId.
        engine_security_group_id(str): The ID of the Amazon EMR Studio Engine security group. The Engine security group allows inbound network traffic from the Workspace security group, and it must be in the same VPC specified by VpcId.
        default_s3_location(str): The Amazon S3 location to back up Amazon EMR Studio Workspaces and notebook files.
        tags(List, optional): A list of tags to associate with the Amazon EMR Studio. Tags are user-defined key-value pairs that consist of a required key string with a maximum of 128 characters, and an optional value string with a maximum of 256 characters. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.emr.studio.init.create(
                ctx,
                name=value,
                auth_mode=value,
                vpc_id=value,
                subnet_ids=value,
                service_role=value,
                user_role=value,
                workspace_security_group_id=value,
                engine_security_group_id=value,
                default_s3_location=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.emr.studio.init.create name=value, auth_mode=value, vpc_id=value, subnet_ids=value, service_role=value, user_role=value, workspace_security_group_id=value, engine_security_group_id=value, default_s3_location=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="emr",
        operation="create_studio",
        op_kwargs={
            "Name": name,
            "Description": description,
            "AuthMode": auth_mode,
            "VpcId": vpc_id,
            "SubnetIds": subnet_ids,
            "ServiceRole": service_role,
            "UserRole": user_role,
            "WorkspaceSecurityGroupId": workspace_security_group_id,
            "EngineSecurityGroupId": engine_security_group_id,
            "DefaultS3Location": default_s3_location,
            "Tags": tags,
        },
    )


async def delete(hub, ctx, studio_id: str) -> None:
    r"""
    **Autogenerated function**

    Removes an Amazon EMR Studio from the Studio metadata store.

    Args:
        studio_id(str): The ID of the Amazon EMR Studio.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.emr.studio.init.delete(ctx, studio_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.emr.studio.init.delete studio_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="emr",
        operation="delete_studio",
        op_kwargs={"StudioId": studio_id},
    )


async def describe(hub, ctx, studio_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns details for the specified Amazon EMR Studio including ID, Name, VPC, Studio access URL, and so on.

    Args:
        studio_id(str): The Amazon EMR Studio ID.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.emr.studio.init.describe(ctx, studio_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.emr.studio.init.describe studio_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="emr",
        operation="describe_studio",
        op_kwargs={"StudioId": studio_id},
    )


async def list_all(hub, ctx, marker: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of all Amazon EMR Studios associated with the account. The list includes details such as ID, Studio Access URL, and creation time for each Studio.

    Args:
        marker(str, optional): The pagination token that indicates the set of results to retrieve. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.emr.studio.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.emr.studio.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx, service_name="emr", operation="list_studios", op_kwargs={"Marker": marker}
    )


async def update(
    hub,
    ctx,
    studio_id: str,
    name: str = None,
    description: str = None,
    subnet_ids: List = None,
    default_s3_location: str = None,
) -> None:
    r"""
    **Autogenerated function**

    Updates an Amazon EMR Studio configuration, including attributes such as name, description, and subnets.

    Args:
        studio_id(str): The ID of the Amazon EMR Studio to update.
        name(str, optional): A descriptive name for the Amazon EMR Studio. Defaults to None.
        description(str, optional): A detailed description to assign to the Amazon EMR Studio. Defaults to None.
        subnet_ids(List, optional): A list of subnet IDs to associate with the Amazon EMR Studio. The list can include new subnet IDs, but must also include all of the subnet IDs previously associated with the Studio. The list order does not matter. A Studio can have a maximum of 5 subnets. The subnets must belong to the same VPC as the Studio. . Defaults to None.
        default_s3_location(str, optional): The Amazon S3 location to back up Workspaces and notebook files for the Amazon EMR Studio. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.emr.studio.init.update(ctx, studio_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.emr.studio.init.update studio_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="emr",
        operation="update_studio",
        op_kwargs={
            "StudioId": studio_id,
            "Name": name,
            "Description": description,
            "SubnetIds": subnet_ids,
            "DefaultS3Location": default_s3_location,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, registry_name: str = None) -> None:
    r"""
    **Autogenerated function**

    Delete the resource-based policy attached to the specified registry.

    Args:
        registry_name(str, optional): The name of the registry. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.schemas.resource.policy.delete(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.schemas.resource.policy.delete
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="schemas",
        operation="delete_resource_policy",
        op_kwargs={"RegistryName": registry_name},
    )


async def get(hub, ctx, registry_name: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the resource-based policy attached to a given registry.

    Args:
        registry_name(str, optional): The name of the registry. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.schemas.resource.policy.get(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.schemas.resource.policy.get
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="schemas",
        operation="get_resource_policy",
        op_kwargs={"RegistryName": registry_name},
    )


async def put(
    hub, ctx, policy: str, registry_name: str = None, revision_id: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    The name of the policy.

    Args:
        policy(str): The resource-based policy.
        registry_name(str, optional): The name of the registry. Defaults to None.
        revision_id(str, optional): The revision ID of the policy. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.schemas.resource.policy.put(ctx, policy=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.schemas.resource.policy.put policy=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="schemas",
        operation="put_resource_policy",
        op_kwargs={
            "Policy": policy,
            "RegistryName": registry_name,
            "RevisionId": revision_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, snapshot_id: str, block_index: int, block_token: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns the data in a block in an Amazon Elastic Block Store snapshot.

    Args:
        snapshot_id(str): The ID of the snapshot containing the block from which to get data.
        block_index(int): The block index of the block from which to get data. Obtain the BlockIndex by running the ListChangedBlocks or ListSnapshotBlocks operations.
        block_token(str): The block token of the block from which to get data. Obtain the BlockToken by running the ListChangedBlocks or ListSnapshotBlocks operations.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ebs.snapshot.block.get(
                ctx, snapshot_id=value, block_index=value, block_token=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ebs.snapshot.block.get snapshot_id=value, block_index=value, block_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ebs",
        operation="get_snapshot_block",
        op_kwargs={
            "SnapshotId": snapshot_id,
            "BlockIndex": block_index,
            "BlockToken": block_token,
        },
    )


async def list_all(
    hub,
    ctx,
    snapshot_id: str,
    next_token: str = None,
    max_results: int = None,
    starting_block_index: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns information about the blocks in an Amazon Elastic Block Store snapshot.

    Args:
        snapshot_id(str): The ID of the snapshot from which to get block indexes and block tokens.
        next_token(str, optional): The token to request the next page of results. Defaults to None.
        max_results(int, optional): The number of results to return. Defaults to None.
        starting_block_index(int, optional): The block index from which the list should start. The list in the response will start from this block index or the next valid block index in the snapshot. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ebs.snapshot.block.list_all(ctx, snapshot_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ebs.snapshot.block.list_all snapshot_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ebs",
        operation="list_snapshot_blocks",
        op_kwargs={
            "SnapshotId": snapshot_id,
            "NextToken": next_token,
            "MaxResults": max_results,
            "StartingBlockIndex": starting_block_index,
        },
    )


async def put(
    hub,
    ctx,
    snapshot_id: str,
    block_index: int,
    block_data: bytes,
    data_length: int,
    checksum: str,
    checksum_algorithm: str,
    progress: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Writes a block of data to a snapshot. If the specified block contains data, the existing data is overwritten. The target snapshot must be in the pending state. Data written to a snapshot must be aligned with 512-byte sectors.

    Args:
        snapshot_id(str): The ID of the snapshot.
        block_index(int): The block index of the block in which to write the data. A block index is a logical index in units of 512 KiB blocks. To identify the block index, divide the logical offset of the data in the logical volume by the block size (logical offset of data/524288). The logical offset of the data must be 512 KiB aligned.
        block_data(bytes): The data to write to the block. The block data is not signed as part of the Signature Version 4 signing process. As a result, you must generate and provide a Base64-encoded SHA256 checksum for the block data using the x-amz-Checksum header. Also, you must specify the checksum algorithm using the x-amz-Checksum-Algorithm header. The checksum that you provide is part of the Signature Version 4 signing process. It is validated against a checksum generated by Amazon EBS to ensure the validity and authenticity of the data. If the checksums do not correspond, the request fails. For more information, see  Using checksums with the EBS direct APIs in the Amazon Elastic Compute Cloud User Guide.
        data_length(int): The size of the data to write to the block, in bytes. Currently, the only supported size is 524288. Valid values: 524288 .
        progress(int, optional): The progress of the write process, as a percentage. Defaults to None.
        checksum(str): A Base64-encoded SHA256 checksum of the data. Only SHA256 checksums are supported.
        checksum_algorithm(str): The algorithm used to generate the checksum. Currently, the only supported algorithm is SHA256.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ebs.snapshot.block.put(
                ctx,
                snapshot_id=value,
                block_index=value,
                block_data=value,
                data_length=value,
                checksum=value,
                checksum_algorithm=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ebs.snapshot.block.put snapshot_id=value, block_index=value, block_data=value, data_length=value, checksum=value, checksum_algorithm=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ebs",
        operation="put_snapshot_block",
        op_kwargs={
            "SnapshotId": snapshot_id,
            "BlockIndex": block_index,
            "BlockData": block_data,
            "DataLength": data_length,
            "Progress": progress,
            "Checksum": checksum,
            "ChecksumAlgorithm": checksum_algorithm,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe(
    hub, ctx, release_label: str = None, next_token: str = None, max_results: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    Provides EMR release label details, such as releases available the region where the API request is run, and the available applications for a specific EMR release label. Can also list EMR release versions that support a specified version of Spark.

    Args:
        release_label(str, optional): The target release label to be described. Defaults to None.
        next_token(str, optional): The pagination token. Reserved for future use. Currently set to null. Defaults to None.
        max_results(int, optional): Reserved for future use. Currently set to null. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.emr.release_label.describe(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.emr.release_label.describe
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="emr",
        operation="describe_release_label",
        op_kwargs={
            "ReleaseLabel": release_label,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )


async def list_all(
    hub, ctx, filters: Dict = None, next_token: str = None, max_results: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves release labels of EMR services in the region where the API is called.

    Args:
        filters(Dict, optional): Filters the results of the request. Prefix specifies the prefix of release labels to return. Application specifies the application (with/without version) of release labels to return. Defaults to None.
        next_token(str, optional): Specifies the next page of results. If NextToken is not specified, which is usually the case for the first request of ListReleaseLabels, the first page of results are determined by other filtering parameters or by the latest version. The ListReleaseLabels request fails if the identity (AWS AccountID) and all filtering parameters are different from the original request, or if the NextToken is expired or tampered with. Defaults to None.
        max_results(int, optional): Defines the maximum number of release labels to return in a single response. The default is 100. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.emr.release_label.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.emr.release_label.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="emr",
        operation="list_release_labels",
        op_kwargs={
            "Filters": filters,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def list_all(
    hub,
    ctx,
    stream_name: str = None,
    stream_arn: str = None,
    max_results: int = None,
    next_token: str = None,
    fragment_selector: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of Fragment objects from the specified stream and timestamp range within the archived data. Listing fragments is eventually consistent. This means that even if the producer receives an acknowledgment that a fragment is persisted, the result might not be returned immediately from a request to ListFragments. However, results are typically available in less than one second.  You must first call the GetDataEndpoint API to get an endpoint. Then send the ListFragments requests to this endpoint using the --endpoint-url parameter.    If an error is thrown after invoking a Kinesis Video Streams archived media API, in addition to the HTTP status code and the response body, it includes the following pieces of information:     x-amz-ErrorType HTTP header – contains a more specific error type in addition to what the HTTP status code provides.     x-amz-RequestId HTTP header – if you want to report an issue to AWS, the support team can better diagnose the problem if given the Request Id.   Both the HTTP status code and the ErrorType header can be utilized to make programmatic decisions about whether errors are retry-able and under what conditions, as well as provide information on what actions the client programmer might need to take in order to successfully try again. For more information, see the Errors section at the bottom of this topic, as well as Common Errors.

    Args:
        stream_name(str, optional): The name of the stream from which to retrieve a fragment list. Specify either this parameter or the StreamARN parameter. Defaults to None.
        stream_arn(str, optional): The Amazon Resource Name (ARN) of the stream from which to retrieve a fragment list. Specify either this parameter or the StreamName parameter. Defaults to None.
        max_results(int, optional): The total number of fragments to return. If the total number of fragments available is more than the value specified in max-results, then a ListFragmentsOutput$NextToken is provided in the output that you can use to resume pagination. Defaults to None.
        next_token(str, optional): A token to specify where to start paginating. This is the ListFragmentsOutput$NextToken from a previously truncated response. Defaults to None.
        fragment_selector(Dict, optional): Describes the timestamp range and timestamp origin for the range of fragments to return. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.kinesis_video_archived_media.fragment.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.kinesis_video_archived_media.fragment.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="kinesis-video-archived-media",
        operation="list_fragments",
        op_kwargs={
            "StreamName": stream_name,
            "StreamARN": stream_arn,
            "MaxResults": max_results,
            "NextToken": next_token,
            "FragmentSelector": fragment_selector,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def detect(hub, ctx, text: str) -> Dict:
    r"""
    **Autogenerated function**

    Determines the dominant language of the input text. For a list of languages that Amazon Comprehend can detect, see Amazon Comprehend Supported Languages.

    Args:
        text(str): A UTF-8 text string. Each string should contain at least 20 characters and must contain fewer that 5,000 bytes of UTF-8 encoded characters.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.comprehend.dominant_language.init.detect(ctx, text=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.comprehend.dominant_language.init.detect text=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="comprehend",
        operation="detect_dominant_language",
        op_kwargs={"Text": text},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def update(hub, ctx, group_id: str, user_rules: List) -> Dict:
    r"""
    **Autogenerated function**

    Replaces the current rules of the specified IP access control group with the specified rules.

    Args:
        group_id(str): The identifier of the group.
        user_rules(List): One or more rules.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.workspace.rule_of_ip_group.update(
                ctx, group_id=value, user_rules=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.workspace.rule_of_ip_group.update group_id=value, user_rules=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="workspaces",
        operation="update_rules_of_ip_group",
        op_kwargs={"GroupId": group_id, "UserRules": user_rules},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def search(
    hub,
    ctx,
    entity_id: str,
    next_token: str = None,
    max_results: int = None,
    namespace_version: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Searches for things associated with the specified entity. You can search by both device and device model. For example, if two different devices, camera1 and camera2, implement the camera device model, the user can associate thing1 to camera1 and thing2 to camera2. SearchThings(camera2) will return only thing2, but SearchThings(camera) will return both thing1 and thing2. This action searches for exact matches and doesn't perform partial text matching.

    Args:
        entity_id(str): The ID of the entity to which the things are associated. The IDs should be in the following format.  urn:tdm:REGION/ACCOUNT ID/default:device:DEVICENAME .
        next_token(str, optional): The string that specifies the next page of results. Use this when you're paginating results. Defaults to None.
        max_results(int, optional): The maximum number of results to return in the response. Defaults to None.
        namespace_version(int, optional): The version of the user's namespace. Defaults to the latest version of the user's namespace. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotthingsgraph.thing.search(ctx, entity_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotthingsgraph.thing.search entity_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotthingsgraph",
        operation="search_things",
        op_kwargs={
            "entityId": entity_id,
            "nextToken": next_token,
            "maxResults": max_results,
            "namespaceVersion": namespace_version,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, account_id: str, bot_id: str) -> None:
    r"""
    **Autogenerated function**

    Deletes the events configuration that allows a bot to receive outgoing events.

    Args:
        account_id(str): The Amazon Chime account ID.
        bot_id(str): The bot ID.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.event_configuration.delete(
                ctx, account_id=value, bot_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.event_configuration.delete account_id=value, bot_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="delete_events_configuration",
        op_kwargs={"AccountId": account_id, "BotId": bot_id},
    )


async def get(hub, ctx, account_id: str, bot_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Gets details for an events configuration that allows a bot to receive outgoing events, such as an HTTPS endpoint or Lambda function ARN.

    Args:
        account_id(str): The Amazon Chime account ID.
        bot_id(str): The bot ID.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.event_configuration.get(
                ctx, account_id=value, bot_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.event_configuration.get account_id=value, bot_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="get_events_configuration",
        op_kwargs={"AccountId": account_id, "BotId": bot_id},
    )


async def put(
    hub,
    ctx,
    account_id: str,
    bot_id: str,
    outbound_events_https_endpoint: str = None,
    lambda_function_arn: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates an events configuration that allows a bot to receive outgoing events sent by Amazon Chime. Choose either an HTTPS endpoint or a Lambda function ARN. For more information, see Bot.

    Args:
        account_id(str): The Amazon Chime account ID.
        bot_id(str): The bot ID.
        outbound_events_https_endpoint(str, optional): HTTPS endpoint that allows the bot to receive outgoing events. Defaults to None.
        lambda_function_arn(str, optional): Lambda function ARN that allows the bot to receive outgoing events. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.event_configuration.put(
                ctx, account_id=value, bot_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.event_configuration.put account_id=value, bot_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="put_events_configuration",
        op_kwargs={
            "AccountId": account_id,
            "BotId": bot_id,
            "OutboundEventsHTTPSEndpoint": outbound_events_https_endpoint,
            "LambdaFunctionArn": lambda_function_arn,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    calculator_name: str,
    data_source: str,
    pricing_plan: str,
    description: str = None,
    tags: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a route calculator resource in your AWS account. You can send requests to a route calculator resource to estimate travel time, distance, and get directions. A route calculator sources traffic and road network data from your chosen data provider.

    Args:
        calculator_name(str): The name of the route calculator resource.  Requirements:   Can use alphanumeric characters (A–Z, a–z, 0–9) , hyphens (-), periods (.), and underscores (_).   Must be a unique Route calculator resource name.   No spaces allowed. For example, ExampleRouteCalculator.  .
        data_source(str): Specifies the data provider of traffic and road network data.  This field is case-sensitive. Enter the valid values as shown. For example, entering HERE returns an error.  Valid values include:    Esri – For additional information about Esri's coverage in your region of interest, see Esri details on street networks and traffic coverage.    Here – For additional information about HERE Technologies's coverage in your region of interest, see HERE car routing coverage and HERE truck routing coverage.   For additional information , see Data providers on the Amazon Location Service Developer Guide.
        description(str, optional): The optional description for the route calculator resource. Defaults to None.
        pricing_plan(str): Specifies the pricing plan for your route calculator resource. For additional details and restrictions on each pricing plan option, see Amazon Location Service pricing.
        tags(Dict, optional): Applies one or more tags to the route calculator resource. A tag is a key-value pair helps manage, identify, search, and filter your resources by labelling them.   For example: { "tag1" : "value1", "tag2" : "value2"}   Format: "key" : "value"  Restrictions:   Maximum 50 tags per resource   Each resource tag must be unique with a maximum of one value.   Maximum key length: 128 Unicode characters in UTF-8   Maximum value length: 256 Unicode characters in UTF-8   Can use alphanumeric characters (A–Z, a–z, 0–9), and the following characters: + - = . _ : / @.   . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.location.route.calculator.create(
                ctx, calculator_name=value, data_source=value, pricing_plan=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.location.route.calculator.create calculator_name=value, data_source=value, pricing_plan=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="location",
        operation="create_route_calculator",
        op_kwargs={
            "CalculatorName": calculator_name,
            "DataSource": data_source,
            "Description": description,
            "PricingPlan": pricing_plan,
            "Tags": tags,
        },
    )


async def delete(hub, ctx, calculator_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a route calculator resource from your AWS account.  This operation deletes the resource permanently.

    Args:
        calculator_name(str): The name of the route calculator resource to be deleted.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.location.route.calculator.delete(ctx, calculator_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.location.route.calculator.delete calculator_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="location",
        operation="delete_route_calculator",
        op_kwargs={"CalculatorName": calculator_name},
    )


async def describe(hub, ctx, calculator_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the route calculator resource details.

    Args:
        calculator_name(str): The name of the route calculator resource.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.location.route.calculator.describe(ctx, calculator_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.location.route.calculator.describe calculator_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="location",
        operation="describe_route_calculator",
        op_kwargs={"CalculatorName": calculator_name},
    )


async def list_all(hub, ctx, max_results: int = None, next_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Lists route calculator resources in your AWS account.

    Args:
        max_results(int, optional): An optional maximum number of results returned in a single call. Default Value: 100 . Defaults to None.
        next_token(str, optional): The pagination token specifying which page of results to return in the response. If no token is provided, the default page is the first page. Default Value: null . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.location.route.calculator.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.location.route.calculator.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="location",
        operation="list_route_calculators",
        op_kwargs={"MaxResults": max_results, "NextToken": next_token},
    )


async def update(
    hub, ctx, calculator_name: str, description: str = None, pricing_plan: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Updates the specified properties for a given route calculator resource.

    Args:
        calculator_name(str): The name of the route calculator resource to update.
        description(str, optional): Updates the description for the route calculator resource. Defaults to None.
        pricing_plan(str, optional): Updates the pricing plan for the route calculator resource. For more information about each pricing plan option restrictions, see Amazon Location Service pricing. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.location.route.calculator.update(ctx, calculator_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.location.route.calculator.update calculator_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="location",
        operation="update_route_calculator",
        op_kwargs={
            "CalculatorName": calculator_name,
            "Description": description,
            "PricingPlan": pricing_plan,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get_all(
    hub,
    ctx,
    query_string: str,
    index_name: str = None,
    aggregation_field: str = None,
    query_version: str = None,
    percents: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Groups the aggregated values that match the query into percentile groupings. The default percentile groupings are: 1,5,25,50,75,95,99, although you can specify your own when you call GetPercentiles. This function returns a value for each percentile group specified (or the default percentile groupings). The percentile group "1" contains the aggregated field value that occurs in approximately one percent of the values that match the query. The percentile group "5" contains the aggregated field value that occurs in approximately five percent of the values that match the query, and so on. The result is an approximation, the more values that match the query, the more accurate the percentile values.

    Args:
        index_name(str, optional): The name of the index to search. Defaults to None.
        query_string(str): The query string.
        aggregation_field(str, optional): The field to aggregate. Defaults to None.
        query_version(str, optional): The query version. Defaults to None.
        percents(List, optional): The percentile groups returned. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.percentile.get_all(ctx, query_string=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.percentile.get_all query_string=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="get_percentiles",
        op_kwargs={
            "indexName": index_name,
            "queryString": query_string,
            "aggregationField": aggregation_field,
            "queryVersion": query_version,
            "percents": percents,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get_all(
    hub,
    ctx,
    crawler_name_list: List = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves metrics about specified crawlers.

    Args:
        crawler_name_list(List, optional): A list of the names of crawlers about which to retrieve metrics. Defaults to None.
        max_results(int, optional): The maximum size of a list to return. Defaults to None.
        next_token(str, optional): A continuation token, if this is a continuation call. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.crawler.metric.get_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.crawler.metric.get_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="get_crawler_metrics",
        op_kwargs={
            "CrawlerNameList": crawler_name_list,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(hub, ctx, project_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Creates a new Amazon Rekognition Custom Labels project. A project is a logical grouping of resources (images, Labels, models) and operations (training, evaluation and detection).  This operation requires permissions to perform the rekognition:CreateProject action.

    Args:
        project_name(str): The name of the project to create.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.rekognition.project.init.create(ctx, project_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.rekognition.project.init.create project_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="rekognition",
        operation="create_project",
        op_kwargs={"ProjectName": project_name},
    )


async def delete(hub, ctx, project_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes an Amazon Rekognition Custom Labels project. To delete a project you must first delete all models associated with the project. To delete a model, see DeleteProjectVersion. This operation requires permissions to perform the rekognition:DeleteProject action.

    Args:
        project_arn(str): The Amazon Resource Name (ARN) of the project that you want to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.rekognition.project.init.delete(ctx, project_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.rekognition.project.init.delete project_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="rekognition",
        operation="delete_project",
        op_kwargs={"ProjectArn": project_arn},
    )


async def describe_all(
    hub, ctx, next_token: str = None, max_results: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    Lists and gets information about your Amazon Rekognition Custom Labels projects. This operation requires permissions to perform the rekognition:DescribeProjects action.

    Args:
        next_token(str, optional): If the previous response was incomplete (because there is more results to retrieve), Amazon Rekognition Custom Labels returns a pagination token in the response. You can use this pagination token to retrieve the next set of results. . Defaults to None.
        max_results(int, optional): The maximum number of results to return per paginated call. The largest value you can specify is 100. If you specify a value greater than 100, a ValidationException error occurs. The default value is 100. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.rekognition.project.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.rekognition.project.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="rekognition",
        operation="describe_projects",
        op_kwargs={"NextToken": next_token, "MaxResults": max_results},
    )
